//
// C++ Interface: DiagWindow
//
//
// Author: Alexander Stein <AlexanderStein@t-online.de>, (C) 2006 - 2016
//
// Copyright: See README file that comes with this source code
//
//

#ifndef DIAGWINDOW_H
#define DIAGWINDOW_H

#include <QVariant>
#include <QPixmap>
#include <QLabel>
#include <QLayout>
#include <QValidator>
#include <QScrollBar>
#include <QGridLayout>
#include <QKeyEvent>
#include <QComboBox>
#include <QPushButton>
#include "tools.h"
#include "Picture.h"
#include "mdichild.h"

class MainWindow;
class Datensatz;
class Marker;

class QPainter;
class QPrinter;
class QFont;
class QMouseEvent;
class QImage;
class QToolButton;

class DiagWindow : public MDIChild
{
    Q_OBJECT

public:
    DiagWindow(Type type = SimpleDiagWindow, MainWindow *MW = 0, 
			   QString Filter = "Data files (*.dat)", QString FileExt = ".dat");
    virtual ~DiagWindow();
	virtual void setData(double **Data, int numRows);
	void addData(double **Data, int numRows);
	void clearData();
	void clear();
	void setShowMouseCross(bool show);
	bool getShowMouseCross() const;
	void setUnits(QString XUnit, QString YUnit);
	void setImage(QImage *Image);
	void setRanges(double xmin, double xmax, double ymin, double ymax);
	void setShowMarkerLabels(bool Show);
	bool getShowMarkerLabels();
	void exportPicture();
	virtual void ClearMarked();
	
public slots:
	virtual void Print(QPrinter &P);
    virtual void Paint();
    virtual void Zoom( QRect * M );
    virtual void HScrolled( int V );
    virtual void VScrolled( int V );
    virtual void ZoomOut();
    virtual void KeyPressed( QKeyEvent * K );
    virtual void KeyPressEvent(QKeyEvent *K);
    virtual void ZoomIn();
    virtual void PictureClicked( QPoint * P );
    virtual void PaintScale(QPainter &P, const QRect &A);
    virtual void PSpektrum(QPainter &P, const QRect &A, bool PrintFN );
	bool writeData(QString FileName = "");
	void getData(double **&Data, int &NPoints);

signals:
    void quit();
	void clicked(double x, double y);
	void clearMarked();

protected:
	void mouseMoveEvent(QMouseEvent *event);
		
	QScrollBar* HScroll;
    QLabel* xStartLabel;
    QLineEdit* xStart;
    QLabel* xStopLabel;
    QLineEdit* xStop;
    QScrollBar* VScroll;
    QLabel* yStartLabel;
    QLineEdit* yStart;
    QLabel* yStopLabel;
    QLineEdit* yStop;
    Picture* Bild;
    QGridLayout* SpektrumLayout;
    QSpacerItem* spacer1;
	QComboBox *IsoBox, *StateBox, *JBox, *vBox, *SimBox, *TUnit, *MolBox, *SourceBox, *RefBox, *CompBox;
    QLineEdit *Evs, *Temp, *LaserF, *WindowS, *WindowE;
	QPushButton *Calc, *PrintB;
	double XMax, XMin, YMax, YMin, XO, YO, XSF, YSF;
	int ScaleMaxLargeSteps, ScaleSmallStepDiv, ScaleNAllowedSteps, *ScaleAllowedSteps;
	int ScaleXHeight, ScaleYWidth, ScaleMinYWidth, ScaleTopOffset;
	QString XUnit, YUnit;
	QFont UnitFont, ScaleFont;
	Datensatz *Daten;
	bool DMarkers, ShowAssignmentsOnTop;
	Marker *marker;
	int AnzahlMarker, nDatenS;
	QFont AssFont;
	QColor SpektColor;
	QColor *CopyColor;
	QToolButton *ZoomB;
	QImage *Image;
	
protected slots:
    virtual void languageChange();

private slots:
	void MouseMovedOverPicture(int x, int y);
	
private:
    QPixmap image0;
    bool ScrollsEnabled, ShowMarkerLabels;
    QDoubleValidator *VEVon, *VEBis, *VWNVon, *VWNBis;

    void init();
    void destroy();

};

#endif // SPEKTRUM_H
