//
// C++ Interface: Spektrum
//
//
// Author: Alexander Stein <AlexanderStein@t-online.de>, (C) 2006 - 2016
//
// Copyright: See README file that comes with this source code
//
//

#ifndef SPEKTRUM_H
#define SPEKTRUM_H

#include <qvariant.h>
#include <qvalidator.h>
#include <qpixmap.h>
#include <QLabel>
#include <QKeyEvent>
#include <QDialog>
#include "TermPlot.h"
#include "tools.h"

#include "DiagWindow.h"
#include "tablewindow.h"
#include <qvarlengtharray.h>

class ElState;
class LineTable;
class Molecule;
class Transition;

class QSpacerItem;
class QAction;
class Picture;
class QScrollBar;
class QLabel;
class QLineEdit;

enum SpektrumTyp {LaserInducedFluorescence, Absorption, NormalizedAbsorption, ThermalEmission};

class SpectList : public TableWindow
{
public:
	SpectList(MainWindow *MW = 0);
	void AutoSLP();
};

class Spektrum : public DiagWindow
{
	Q_OBJECT
			
public:
    Spektrum(MainWindow *MW = 0);
    ~Spektrum();
	
    IntProg* SearchLongestProg(Marker *M1);
    void editFind();
    bool readData(QString DateiName);
	bool writeData(QString FileName = "");
	void setFileName(QString FileName);
    void DisplayMarkers();
	void GetMarker(int &AnzahlMarker, Marker *&marker);
	int getNLines();
	void ShowMarker();
    void FitLine( double LP );
    void PictureClicked( QPoint * P );
    void SearchLines();
    void FoundB();
    void FoundF();
    void ShowFound();
	void FocusSelected();
	void AcceptAssignments();
    void CSLSettings();
    void ClearMarked();
    void TTransition(int NumWFPoints);
	void FindProgressions();
	void FindLinesFromTable(LineTable *LTab);
	void SearchProgression();
    void FindSat();
	void ContinueBand();
	void ContinueProgressions();
    void ShowTermPlot();
    void ShowLineTable();
    void BRauschen();
	void SetLaserFrequency(double Frequency);
	double GetLaserFrequency();
	Marker *GetLaserLine();
	void SingleSLP();
	void MultiSLP();
	void AutoSLP();
	void MSLP(IntProg **&P, int &N);
	void FindEmissionLines(Molecule *mol, int Iso, ElState *EState, double MinH, double Tol, int MLPB);
	void IsLI();
	void addMarker(bool autoaccept = false);
	void MarkProgression(IntProg &P);
	void WriteIntensDist();
	int *getIntensDist(int numPoints, double minI, double maxI);
	void getMinMaxIntensities(double &MinI, double &MaxI);
	void getRMPH(double &Rauschen, double &MinPeakHeight);
	void AssignBands();
	void assignBand();
	void assignBandByDoubletPartners();
	Marker *getMarker(double SearchEnergy, double Tol = 0.0);
	void AverageWeighted(int NSpectra, QString *SpektFiles, int NLines, int **Lines, ElState *LS, 
						 ElState *US, int Iso, int Comp, int vs, int Js);
	void cut(double *Start, double *End, int NAreas);
	void cutAssignedLines();
	void cutStrongLines();
	double getMaxR();
	void getProgProfile(ElState *UState, ElState *LState, int Iso, int vs, int Js, int Comp, 
					    int *&vss, int *&Jss, double *&I, int &N);
	bool addByBand(ElState *UState, ElState *LState, int Iso, int vs, int vss, int Comp, double T, 
				   double SearchRadius, double Resolution, double **Result);
	bool addByProgression(ElState *LState, int Iso, int Js, int Comp, Spektrum *RSpektrum, 
						  double Ri, double Ra, double Resolution, int RIso, int RJs, int RComp,
						  double **Result);
	bool addByProgFCF(ElState *UState, ElState *LState, int Iso, int vs, int Js, int Comp, double T, double SearchRadius, 
                      double Resolution, double **Result, int NumWFPoints, double uE = 0.0, double **uWF = 0, double* TStr = 0, int NC = 0);
	void autoAddByProgFCF(ElState *UState, ElState *LState, int Iso, int Comp, double T, 
                          double SearchRadius, double Resolution, int NumWFPoints);
	bool add(double *OffSet, double *ScaleFakt, int N, double Ri, double Ra, double Resolution, 
			 double **Result);
	void optimizeWeighting(double **OffSet, double *TE, int NOff, int NTE, double *WF);
	void normalize(Spektrum *refSpect);
	void SetShowAssignmentsOnTop(bool V);
	bool GetShowAssignmentsOnTop();
	void LineTableSaved(ElState *UState, ElState *LState);
	int getType();
	void setType(int newType);
	void setData(double **Data, int numRows);
	
	inline double getMinH()
	{
		return MinPeakHeight;
	}

signals:
	void SpectrumChanged(Spektrum *This);
	
private:
    int SML(IntProg &P, int &I, int &J, double &T, double &E, int veu, double ***ELU);
	bool isLI(const int &I, int &v, const int &J, double T, double &d, double ***ELU, int veu);
	bool cbSat(int &I, int &v, int &J, Marker &L, Marker **&M, int &N, Marker *&ML);
	void GetMGMH();
	void GetMA(Marker **&A, int &N);
	int Assignvs(IntProg &P);
	bool excitedLevel(IntProg &P);
	void cFQS(IntProg &P);
	void chkDoubletts(IntProg &P);
	void eNGL(IntProg &P, Marker **&AL, int NAL, double ***ELU);
	void getMarkedLines(Marker **&M, int &N);
	void getUnassignedLines(Marker **&M, int &N);
	void FokusProg(Marker **M, int &N);
	void UpdateProg(IntProg &P);
	bool chkPrep();
	double getAUT(IntProg &P);
	void rSortP(IntProg &P);
	void ASLP(QString Filename);
	void continueBand();
	void Paint();
	int PQC();	
	
	Marker *LaserLinie, MBeginn, MStop, **GMarker;
	int LPI, DC, numLineTablesAT, Type;
	int AssignmentStatus;
	bool useIntensities;
	double ST, GMH, MinPeakHeight, Rauschen, LaserFrequency, LaserFTol;
	Progressions Prog;
	AProg *Assigned;
	Band *band;
	LineTable **LineTablesAT;
};

class SelectLineTableDialog : public QDialog
{
	Q_OBJECT
	
public:
	SelectLineTableDialog(MainWindow *MW = 0);
	LineTable *getLineTable();
	
private slots:
	void MoleculeChanged(int n);
	void TransitionChanged(int n);
	
private:
	Molecule *Mol;
	Transition *Tr;
	MainWindow *MW;
	
	QComboBox *MolB, *TranB, *LTB;
	QPushButton *OK, *Cancel;
};

class FELDialog : public QDialog
{
	Q_OBJECT
	
public:
	FELDialog(MainWindow *MW, double initMinH);
	Molecule *getMol();
	ElState *getState();
	int getIso();
	int getMLPB();
	double getTol();
	double getMinH();
	
	static bool DataA(MainWindow *MW);
	static bool DataA(Molecule *Mol);
	static bool DataA(Molecule *Mol, ElState *State);
	static bool DataA(Molecule *Mol, ElState *State, int Iso);
	
private slots:
	void MolChanged(QString Name);
	void StateChanged(QString Name);
	
private:
	Molecule *Mol;
	ElState *State;
	MainWindow *MW;
	QComboBox *MolBox, *StateBox, *IsoBox;
	QLineEdit *MLPBEdit, *TolEdit, *MinHEdit;
	QPushButton *OK, *Cancel;
};

#endif // SPEKTRUM_H
