//
// C++ Interface: adddialog
//
//
// Author: Alexander Stein <AlexanderStein@t-online.de>, (C) 2009 - 2016
//
// Copyright: See README file that comes with this source code
//
//

#ifndef ADDDIALOG_H
#define ADDDIALOG_H

/**
	@author Alexander Stein <AlexanderStein@t-online.de>
*/

#include "DiagWindow.h"
#include "Spektrum.h"

#include <QMenu>
#include <QToolButton>

class QComboBox;
class QLineEdit;
class QLabel;
class QPushButton;
class QRadioButton;
class QCheckBox;
class QImage;
class QTextEdit;

class MainWindow;
class DiagWindow;
class FitData;
class TermTable;
class AddDialog;
class AddSpectrum;
class ResidualFit;
struct TermEnergy;

enum autoUpdateResult {autoUpdateSuccess, autoUpdatePartialFailure, 
					   autoUpdateCompleteFailure, autoUpdateInsufficientData};
enum MarkedPeakLevelState {hiddenSaved = -1, hiddenNew, savedAssignment, newAssignment,
						   markedSaved, savedToDelete, newToDelete};

struct MarkedPeak
{
	int JPos, EPos, LevelState, Row, StateNum, SplineNum;
	bool CalcFromProgression, CorrectVibLevel;
};

class DoubleAddSpectrum : public Spektrum
{
	Q_OBJECT
	
public:
	DoubleAddSpectrum(AddSpectrum *addSpectrum, MainWindow *MW);
    virtual ~DoubleAddSpectrum();
	double getDiff();
	double getSNR();
	
private slots:
	void optimize();
	
	inline void showPMenu(QPoint *P)
	{
		PMenu->popup(*P);
	}
	
private:
	AddSpectrum *addSpect;
	QMenu *PMenu;
};


class AddSpectrum : public DiagWindow
{
	Q_OBJECT
	
public:
	AddSpectrum(MainWindow *MW, AddDialog *D);
	~AddSpectrum();
	void setData(double **nData, int NJ, int JStart, int JStep, int NE, Spektrum *Source, 
				 ElState *State, int Comp, int Iso, int v, TermTable *TT, double Ri, 
				 double Ra, double Res, double *UEnergy, int *vAssign, int *cAssign);
	void getMarked(int &N, MarkedPeak *&MarkedLevels);
	void maximizeDAdd(double Diff);
	int AutoUpdateLevels(bool fullyAutomatic);
	
	inline int getSpectrumType()
	{
		return type;
	}
	
public slots:
	virtual void PSpektrum(QPainter &P, const QRect &A, bool PrintFN);
	virtual void KeyPressEvent(QKeyEvent *K);
	void acceptAutoUpdateResult();
	void rejectAutoUpdateResult();
	
private slots:
	void mark(QRect *R);
	void removeLevels();
	void updateLevels();
	void hideLevels();
	void showLevels();
	void markLevels();
	void setEnergyReference();
	void ImageClicked(double dJ, double E);
	void AssignmentsAccepted(FitData *CFD);
	void stateChanged();
	void drawDAddSpectrum(double dRes, bool newState, int SplineNum = -1, int JMin = -1, int JMax = -1);
	void fillTEStruct(TermEnergy &TE, int &FDatNr, int JPos, int EPos);
	void addLevel(TermEnergy &TE, int &FDatNr, int JPos, int EPos);
	
	inline void autoUpdateLevels()
	{
		AutoUpdateLevels(false);
	}
	
	inline void showPMenu(QPoint* P)
	{
		if (!ZoomB->isChecked())
		{
			if (NMBAUL == -1) PMenu->popup(*P);
			else AutoUpdateMenu->popup(*P);
		}
	}
	
private:
	void calcEJ(double &rE, int &rJ, int J, int P);
	int getPeakTop(int JPos, int StartPos);
	
	FitData *FitD, **PFitD;
	MarkedPeak *marked;
	int NMarked, type, Comp, TDC, v, Iso, NP, NJ, JStart, JStep, NC, NMBAUL;
	int NPert, *NCP, *NvP, *NJP, *EShift, NOC, NOIso, NOJ, NOv, *vAss, *CAss;
	double **Data, Ri, Ra, Res, *****PTData, ****OTData, *UEn;
	bool **UCP;
	TermTable *TT, *OffTT, *PTT[50];
	ElState *State;
	Spektrum *Source;
    AddDialog *Dialog;
	DoubleAddSpectrum *DAddSpect;
	ResidualFit *ResFit;
	QMenu *PMenu, *AutoUpdateMenu;
	QAction *removeLevelsAct, *updateLevelsAct, *hideLevelsAct, *showLevelsAct;
	QAction *markLevelsAct, *autoUpdateAct;
};


class AddDialog : public QWidget
{
	Q_OBJECT
	
	public:
    	AddDialog(MainWindow *MW, Spektrum *Spekt);
    	~AddDialog();
		void enableOptimize(bool enable);
		
	private slots:
		void Apply();
		void MolChanged(QString Name);
		void MethodChanged();
		void UpdateSpektList();

	private:
		QComboBox *MolBox, *USBox, *LSBox, *IsoBox, *RIsoBox, *CompBox, *SpektBox, *RSpektBox;
		QComboBox *RCompBox, *TUnit;
		QLineEdit *L1E, *L2E, *TempE, *RiE, *RaE, *ResE, *RJsE;
		QLabel *L1, *L2, *L14;
		QPushButton *ApplyB, *CloseB;
		QRadioButton *AddByBand, *AddByProg, *AddByProgFCF;
		QCheckBox *AutoAnalyse, *DrawImage, *UpdateAllVs;
		MainWindow *MW;
		AddSpectrum *Image;
		Spektrum *Spekt;
		TermTable *TT, *LT;
		double ****TData, **IData;
		int NJ, NC, JStart, JStep, NumIso;
		bool Marked;
		QTextEdit *LogWindow;
};

#endif
