//
// C++ Interface: FitData
//
//
// Author: Alexander Stein <AlexanderStein@t-online.de>, (C) 2008 - 2016
//
// Copyright: See README file that comes with this source code
//
//

#ifndef FITDATA_H
#define FITDATA_H

#include "tablewindow.h"

#include <QDialog>
#include <QCheckBox>

class TableLine;
class LineTable;
class TermEnergy;

class QListWidget;

enum {fdcIso, fdcv, fdcJ, fdcvs, fdcJs, fdcSource, fdcProg, fdcFile, fdcEnergy,
        fdcUncert, fdcObsCalc, fdcDevR};

bool sortByProg(QTableWidget *Tab, int n, int m);
bool sortIvJFreqF(QTableWidget *Tab, int n, int m);
bool SortvsIvJ(QTableWidget *Tab, int n, int m);
bool sortIefJFreq(QTableWidget *Tab, int n, int m);
bool sortIefJFreqv(QTableWidget* Tab, int n, int m);
bool sortbyDeviation(QTableWidget *Tab, int n, int m);
bool sortbyDevR(QTableWidget *Tab, int n, int m);
bool sortforTFGS(QTableWidget* Tab, int n, int m);

enum {eLevel, fLevel};

class TSDialog : public QDialog
{
	Q_OBJECT
	
	public:
		TSDialog(QWidget *parent, Molecule *M, ElState *S);
		~TSDialog();
		
		int getNumSelected();
		LineTable *getSelected(int n);
		bool getKeepOther();
		
		inline bool getUpdateSA()
		{
			return UpdateSA->isChecked();
		}
		
	private slots:
		void addTable();
		void removeTable();
		void OK();
		
		inline void KeepOtherChanged(bool Checked)
		{
			UpdateSA->setEnabled(Checked);
		}
		
	private:
		LineTable **Tables;
		int NTables;
		
		QListWidget *List;
		QComboBox *Box;
		QCheckBox *KeepOther, *UpdateSA;
		QPushButton *Add, *Remove, *OKB, *Cancel;
};


class vAssignDialog : public QDialog
{
	Q_OBJECT
	
public:
	vAssignDialog(MainWindow *parent, ElState *S);
	bool getSelectedTables(TermTable **&T, int &N, double &Tol, double &DATol);
	
private slots:
	void addSource();
	void removeSource();
	
private:
	QListWidget *List;
	QComboBox *Box;
	QPushButton *Add, *Romove, *OK, *Cancel;
	QLineEdit *Tolerance, *DATolerance;
	QList<int> L1, L2;
	
	ElState *State;
	int NT, ND;
};


class SourceOffsetDialog : public QDialog
{
	Q_OBJECT
	
public:
	SourceOffsetDialog(QStringList &Names, double *Offsets, MainWindow *parent = 0);
	
private slots:
	void ChangeIndex(int Index);
	void Save();
	
private:
	int lastIndex;
	double *offsets;
	QLineEdit *OffsetE;
};


struct LevelComb
{
	int ef, iso, v;
};


class FitData : public TableWindow
{
	Q_OBJECT
	
	public:
		FitData(ElState *State = 0, MainWindow* MW = 0, Molecule* M = 0);
		~FitData();
		
		void getData(TableLine *&Lines, int &NLines, int JD = -1, int F = -2, int v = -2, int mJ = 0, int Iso = -1);
		void getData(TableLine *&Lines, int &NLines, int *&RowN,
					 bool sortFuncs(QTableWidget *Tab, int n, int m), int *mv = 0, int mJ = 0);
		void setData(TableLine *Lines, int NLines);
		void addData(TableLine *Lines, int NLines);
		ElState *getElState();
		void setElState(ElState *State);
		int getMaxJ();
		int getMaxv();
		void updateData();
		void updateRow(TableLine *Line);
		void updateEnergy(int N, int *r, double *Energy);
		void updateLevels(int N, int *r, double *Energy, int *vs, int *vss);
		void setDev(double *dev, double *DevR);
		void setDev(double *dev, int *RowN, int N);
		void setRWErr(double *RWErr);
		void setWeightFactors(int N, int *Rows, double *Factors);
		void setMCSEnergies(double *MCSE, int *Rows, int N);
		void setUncertainty(int *Rows, double *Uncertaintys, int N);
		void findWrongData();
		void deleteRows(int *rows, int N);
		void AddRow();
		void DeleteRows();
		void removeDataFSource();
		void removeSingleLines();
		bool isDataAvailable();
		bool readData(QString Filename);
		bool writeData(QString Filename = "");
		bool writeExPotFitInput(QString fileName);
		bool writeTFGS(QString Filename);
		int addMarkedLevel(TermEnergy &TE, Spektrum *Source);
		void removeMarkedLevel(TermEnergy &TE, Spektrum *Source);
		void clearMarkedLevels();
		void RemoveDoubled();
		void Assign_v(double ****TE, int NC, int NI, int NJ, int *Nv, int *IsoTrans, int *CompTrans, double Tolerancy, 
					  double DoublAssTol);
		void getNumLevels(int **&LNum, int &NumIso, int &NumComp, int type = 0);
		void getUncertaintyStats(QList<double> &Uncert, QList<int> &Numbers);
		double getUncertaintyOfvibLevel(int v, int I, Spektrum *Source);
		int getNumProgressions(int *mv = 0, int mJ = 0);
		int getNumLines(int *mv, int mJ);
		int getNumLines(int JD, int F=-2, int v = -2, int mJ = -1, int Iso = -1); 
		void getav(int &nv, int *&v);
		QList<int> getaFC();
		void getSourceOffset(QStringList &Names, double *&Offsets);
		void setSourceOffset(QStringList &Names, double *Offsets);
		QStringList getSources();
		void selectDataFSource(QString Source);
		void setUncertainty(double Uncertainty, bool Min);
		void setFC(int FC);
		int getNumLines();
		void getavIso(int *&Iso, int &NIso);
		QList<LevelComb> getAvLevelComb(int iso, int comp, int vmin, int vmax);
        bool checkSourceConnections();

        inline bool checkAllConnections()
        {
            bool result = true;
            if (!checkSourceConnections()) result = false;
            if (!TableWindow::checkAllConnections(fdcFile)) result = false;
            return result;
        }

        void shrinkAllSpectRefs()
        {
            TableWindow::shrinkAllSpectRefs(fdcFile);
        }
		
		inline void sortIvJF()
		{
			sortTab(heapSort(sortIvJFreqF));
		}
		
		inline void sortvsIvJ()
		{
			sortTab(heapSort(SortvsIvJ));
		}
		
		inline void sortProg()
		{
			sortTab(heapSort(sortByProg));
		}
		
		inline void sortByDev()
		{
			sortTab(heapSort(sortbyDeviation));
		}
		
		inline void sortByDevRatio()
		{
			sortTab(heapSort(sortbyDevR));
		}

        enum TableColumns {Iso, vss, Jss, vs, Js, Source, PN, File, WN, err, dev, devR, WeightF};
		
	signals:
		void AssignmentsAccepted(FitData*);
		
	private:
		void getData(TableLine *Lines, int *SA, int NLines, int *RowN = 0, int mv = -1, int *Mv = 0, 
					 int mJ = 0, int JD = -1, int F = -2, int Iso = -1);
		void sortTab(int *SArray);
		
		ElState *State;
		LineTable **Sources;
		int NMarkedLevels, NSources, *FC, lRow, NSourceOffset;
		QString *SourceOffsetNames;
		double *SourceOffset;
		QPixmap *NewPix;
};

#endif // FITDATA_H
