//
// C++ Interface: FitObject
//
//
// Author: Alexander Stein <AlexanderStein@t-online.de>, (C) 2014 - 2016
//
// Copyright: See README file that comes with this source code
//
//

#ifndef FITOBJECT_H
#define FITOBJECT_H

class FitObject
{
public:
	FitObject(int NPar);
	FitObject(int NPar, double *x, double *y, double *Sig, int N);
	~FitObject();
	void addDataPoint(double x, double y, double Sig);
	double LevenbergMarquardt(int MaxIt, double MinImp);
	virtual void setData(double *x, double *y, double *Sig, int N);
		
protected:
	virtual void getDerivatives(double **deriv);
	virtual void getCalcY(double *Ycalc);
	virtual void getPar(double *Par);
	virtual void setPar(double *Par);
	virtual void updatePar(double *C);
	
	double *X, *Y, *sig;
	int nData, nPar;
	
private:
	void destroyData();
};

class Lorentzian : public FitObject
{
public:
    Lorentzian();
	Lorentzian(double *x, double *y, double *Sig, int N);
	double getE();
    void setData(double* x, double* y, double* Sig, int N);
	
protected:
	void getDerivatives(double **deriv);
	void getCalcY(double *YCalc);
	void getPar(double *Par);
	void setPar(double *Par);
	void updatePar(double *C);
	
private:
	double A, E, Gamma, AWF, EWF, GWF;
};

#endif // FITOBJECT_H
