//
// C++ Interface: linetable
//
//
// Author: Alexander Stein <AlexanderStein@t-online.de>, (C) 2098 - 2016
//
// Copyright: See README file that comes with this source code
//
//

#ifndef LINETABLE_H
#define LINETABLE_H

#include "elstate.h"
#include "tablewindow.h"

#include <qmessagebox.h>

class Molecule;
class MainWindow;
class Marker;
class Progression;
class IsoTab;
class Spektrum;
class Progression;
class LineTable;

class QComboBox;
class QPushButton;
class QStringList;
class QTableWidgetItem;
class QTableWidgetSelectionRange;
//class QList<QTableWidgetSelectionRange>;


enum TableCols {CPN, Cvs, CJs, Cvss, CJss, CF, CWN, Cerr, CIso, CFile, CSNR, CDev, CC,
				CFCF, CEUp, CEav, CEUma, CEdJ, CCalc, COmC};
#define TableNormCols 13

struct TableLine
{
	int vs, Js, vss, Jss, Iso, PN, Row, SourceN, FC, nDig;
	double WN, err, dev, DevR, SNR, WeightFact;
	QString File, SourceName;
	LineTable *LTab;
	bool isTE, isViewn, isSelected;
};

struct TermEnergy
{
	int v, J, Iso, PN, FC, nDig, Row;
	double E, err, dev, DevR, WeightFactor;
	bool ef;
	QString File;
};

class FSPDiag : public QWidget
{
	Q_OBJECT
	
	public:
		FSPDiag(MainWindow *MW, Progression P);
		~FSPDiag();
		
	private slots:
		void search();
		
	private:
		QComboBox *LTBox;
		QPushButton *SearchB, *CloseB;
		Progression *P;
		MainWindow *MW;
};

class QuestionBox : public QMessageBox
{
	Q_OBJECT
	
	public:
		QuestionBox(QString Caption, QString Question, QWidget* parent = 0);
		void addButton(QString Text);
		int getResult();
	
	private slots:
		void b0();
		void b1();
		void b2();
		void b3();
		void b4();
				
	private:
		int Result, NButtons;
};

struct vsOListElement
{
	int vs, Js, Iso, curMaxOffset;
	vsOListElement *next;
};

class LineTable : public TableWindow
{
	Q_OBJECT
	
public:
    LineTable(MainWindow *MW = 0, Molecule *M = 0, Transition *T = 0);
    ~LineTable();
	void setTransition(Transition *T);
	Transition *getTransition();
	int getAnzahlLinien();
    void getLines(int **Zuordnung, double *Energien, double *Uncertainties);
	void getLines(const QString &Filename, double **Lines, int *numLines);
	void getLines(TableLine *&L, int &N);
	void getSortedLines(TableLine *&L, int &N, int SortOrder = 0);
	void getgoodLines(int &N, TableLine *&L, int *mv = 0, int mJ = 0);
	void setUncertainty(int *RowNumbers, double *NewUncertainty, int NLines);
	int getNgL(int *mv = 0, int mJ = 0);
	int getNgTE(int *mv = 0, int mJ = 0);
	void getgoodTE(int &N, TermEnergy *&TE, int *mv = 0, int mJ = 0);
	int getMaxvs();
	int getMaxvss();
	int getMaxJs();
	int getMaxJss();
	Progression getSelectedProgression();
	void findSimilarProgression(Progression P);
	void getObsIso(bool *ObsIso, int NIso);
	void getProgressions(int &N, Progression *&P);
	bool writeData(QString Filename = "");
	bool readData(QString Filename = "");
	void addData(QString **Data, int NR, int NC);
	void setData(int NRows, int nCols, QStringList &HHeader, QTableWidgetItem ***Data);
	void splitTable();
	void WriteTFGS(QString FileName = "", int vso = 1000, vsOListElement *vsOList = 0);
	void W2AI(QString sldc = "");
	bool writeExcPotFitInput(QString FileName);
	void Assignvs();
	void AssignFC();
	void AddMarked(int AnzahlMarker, Marker *marker, Marker *LaserLine, QString SpektFile);
	void AcceptAssignments(QString SpektFile, bool accept);
    void MarkSelected();
	void MarkLines(int *Iso, int *vs, int *Js, double *WN, int N);
	void findErrors();
	
	inline void MarkLines(int *R, int N)
	{
		TableWindow::MarkLines(R, N);
	}
	
	inline ElState *getElState()
	{
		return (transition != 0 ? transition->getUpperState() : 0); 
	}
	
	void TakeOnChanges();
    void TestProgressions(int NumWFPoints);
    void DeleteRows();
	void deleteRows(int *rows, int N);
	void cutRows(int &numRows, int &numColumns, QString **&Data);
    bool ShowUpTerm();
    void ShowCalcRelInt(int NumWFPoints);
	void ShowGSDeviations();
    void FindBigDiff();
	void ShowWeakProgressions();
    TableWindow *ShowUpTermTable();	
	void Updatevs(int *nvs);
	void RemoveDoubled();
	void sortUpTermIvJ();
	void SortProg();
	void SortIJvP();
	void SortIvPJ();
	void SortFPInt();
	void SortSpectrum();
	void SortfRemDoubled();
	void SetError();
	void SetError(int NL, int *PN, int *vss, int *Jss, double *Err);
	void SetError(int NL, int *PN, int *vss, int *Jss, QString *Err);
	void DeleteRows(int NL, int *PN, int *vss = 0, int *Jss = 0);
	void SetPN();
	void Shiftvup();
	void Shiftvdown();
	void ShiftJup();
	void ShiftJdown();
	void Shiftvsup();
	void Shiftvsdown();
	void ShiftIso();
	void SetvssAscending();
	void Delete();
	void setvs();
	void setFC(QString nF);
	void sortbyvs();
	void getKnownLevels(int NI, int &mvs, int &mvss, int &mJs, int &mJss, bool ***&uL, bool ***&lL);
	void setMolecule(Molecule *mol);
	void getSelData(int *&Js, double *&E, int &N);
	void getViewnE(int *&Js, double *&E, int &N);

    inline bool checkAllConnections()
    {
        return TableWindow::checkAllConnections(CFile);
    }

    inline void shrinkAllSpectRefs()
    {
        TableWindow::shrinkAllSpectRefs(CFile);
    }
	
public slots:
	void updateMarker(Spektrum *Spectrum);
    //void printf(const char* arg1);
	
private slots:
	void TabSelChanged();
	
signals:
	void DataChanged();
	
private:
	void sortTab(int *SortArray);
	void UpdateMarker(Spektrum *Spectrum = 0, int numLines = -1, int *Lines = 0, bool remove = false);
	
	Transition *transition;
	int NR, lRow, MaxPN, mvs, mJs, mIso, NpProg, NpL, *SelJs, NSel, *SO, NSO;
	double Error, OvError, *SelE;
	QString InFile, mSpectrum;
	QStringList HeaderLabels;
	TableWindow *termTable;
	IsoTab *Iso;
	QList<QTableWidgetSelectionRange> SelR;
};

bool isnSPG(QTableWidget *Tab, int n, int m);
bool sortUtIvJ(QTableWidget *Tab, int n, int m);
bool sortIJvP(QTableWidget *Tab, int n, int m);
bool sortIvPJ(QTableWidget *Tab, int n, int m);
bool sortFPInt(QTableWidget *Tab, int n, int m);
bool sortIJvFreq(QTableWidget *Tab, int n, int m);
bool sortByvs(QTableWidget *Tab, int n, int m);
bool sortForSPN(QTableWidget *Tab, int n, int m);
bool sortBySpectrum(QTableWidget *Tab, int n, int m);
bool sortfRemDoubl(QTableWidget *Tab, int n, int m);
bool sortByFrequency(QTableWidget *Tab, int n, int m);

bool isnSPG(TermEnergy& T1, TermEnergy &T2);

#endif
