//
// C++ Interface: MonteCarloSim
//
//
// Author: Alexander Stein <AlexanderStein@t-online.de>, (C) 2011 - 2016
//
// Copyright: See README file that comes with this source code
//
//

#ifndef MONTECARLOSIM_H
#define MONTECARLOSIM_H

#include <QPlainTextEdit>
#include <QDialog>
#include <QTableWidget>
#include <QProcess>

#include "tablewindow.h"

class Potential;
class FitData;
class MainWindow;

class QLineEdit;
class QRadioButton;
class QPushButton;
class QFile;
class QTextStream;
class QMessageBox;
class QComboBox;
class QLabel;
class QTimer;
class QTcpServer;
class QTcpSocket;
class QCheckBox;
class QTime;

struct FitResult
{
	FitResult();
	void initialize();
	bool better(FitResult *Result2);
	bool equal(FitResult *Result2);
	bool betterEqual(FitResult *Result2);
	bool isValid();
	
	int NBad, NBadPAL, Index, ProcessNum;
	double Sigma, FQS_Bad, ParValue, LRelParDev;
	bool isStartPot;
};

struct IncludePot
{
	QString FileName;
	int pos1, pos2;
};


class ProcessView : public QPlainTextEdit
{
	Q_OBJECT
	
public:
	ProcessView(QWidget *parent, QString Directory, int ProcessNumber);
	~ProcessView();
	void start(int FitNum);
	void stop();
	void setData(QString FitProg, QString FitData, QString FitInFile1, QString FitInFile2, 
				 QString FitOutInFile, QString FitInFileName, QString FitOutFileName, 
			     QString BadListName, QString PotTempFileName, Potential *Pot);
	void setPotential(Potential *Pot);
	Potential *getPotential(int N = -1);
	double getSigma(int N = -1);
	QString getPotFileName(int N);
	bool isPotAvailable(int N);
	QStringList getBadList(int N = -1);
	bool isRunning();
	void saveResults(int ResultNum);
	
signals:
	void finished(int ProcessNumber, int LP);
	
private slots:
	void read();
	void ProcessFinished(int exitCode, QProcess::ExitStatus ExitStatus);
	void testWorking();
		
private:
	void finish();
	
	int Status, ProcessNum, FitNum;
	double lSigma, oSigma;
	bool working, killed;
	QProcess *Process;
	QString FitProg, FitData, FitInFile1, FitInFile2, FitOutInFile, FitInFileName;
	QString FitOutFileName, Directory, BadListName, PotTempFileName, PotFileName; 
	Potential *Pot;
	//QTextStream *LogStream;
	QTimer *Timer;
};


class MCFSettingsDialog : public QDialog
{
	Q_OBJECT
	
public:
	MCFSettingsDialog(MainWindow *MW);
	void getResults(QString &WorkDir, QString &WorkDir2, QString &WorkDir3, QString &ResultLogFile, QString &FitProg, QString &FitData, 
					QString &FitInFile1, QString &FitInFile2, QString &FitOutInFile, QString &FitInFileName, QString &FitOutFileName, 
				    QString &BadListName, QString &InitPotDir, QString &PotentialName, QString &PotTempFileName, int &Method, 
				    double &Prec, double &RStart, double &RStart2, double &RStop, double &RStop2, double &Step, double &Step2, 
				    int &Parameter, int &Parameter2, QString &serverIP, bool &internalFit, bool &Improvement, int &numParIt, 
				    Potential *&Pot);
	
private slots:
	void statusChanged(bool newStatus);
	void sWorkDir();
	void sWorkDir2();
	void sWorkDir3();
	void sResultLogFile();
	void sInitPotDir();
	void sFitProgD();
	void sFitDataD();
	void sFitInFile1D();
	void sFitInFile2D();
	void sFitOutInFileD();
	void sFitInFileNameD();
	void sFitOutFileNameD();
	void sBadListNameD();
	void sPotentialName();
	void sPotTempFileNameD();
	void accept();
	void potentialChanged();
	void MethodChanged(bool s2DChecked);
	void intExtFitProgChanged(bool internal);
	void numParItChanged();
	
private:
	QLineEdit *FitProgE, *FitDataE, *FitInFile1E, *FitInFile2E, *FitOutInFileE, *WorkDirE2, *WorkDirE3;
	QLineEdit *FitInFileNameE, *FitOutFileNameE, *BadListNameE, *PotTempFileNameE;
	QLineEdit *PrecE, *RStartE, *RStopE, *StepE, *ResultLogFileE, *WorkDirE, *InitPotDirE;
	QLineEdit *RStartE2, *RStopE2, *StepE2, *PotentialNameE, *ServerIP, *numParIt;
	QRadioButton *searchMin, *estimateUncert, *search2DPlane, *internalFitRoutine, *externalProgram;
	QPushButton *sFitProgB, *sFitDataB, *sFitInFile1B, *sFitInFile2B, *sFitOutInFileB;
	QPushButton *sFitInFileNameB, *sFitOutFileNameB, *sBadListNameB, *sPotTempFileNameB;
	QPushButton *OK, *Cancel, *sResultLogFileB, *sWorkDirB, *sWorkDirB2, *sWorkDirB3;
	QPushButton *sInitPotDirB, *sPotentialNameB;
	QComboBox *ParameterB, *ParameterB2, *PotBox;
	QLabel *WorkDirL, *FitProgLabel, *DataFileLabel, *inputFile1Label, *inputFile2Label, *inffoutLabel, *inFileLabel, *FitOutFileLabel;
	QLabel *BadListNameLabel, *PotTempFileNameLabel, *PotentialLabel;
	QCheckBox *isClient, *withImprovement;
	MainWindow *MW;
	QString Dir;
	QIntValidator *NumParItValid;
	Potential *StartPot;
};

class FieldWindow : public QWidget
{
public:
	FieldWindow(int ***Field, int N1, int N2);
	void update(FitResult *Results, int NResults, int *Pos1, int *Pos2, int MaxParFits);
	
protected:
	void paintEvent(QPaintEvent *P);
	
private:
	int ***Field, NxPos, NyPos, Step, NResults, *xPos, *yPos, MaxParFits;
	FitResult *Results;
};

class MonteCarloSim : public TableWindow
{
	Q_OBJECT
	
public:
    MonteCarloSim(MainWindow *MW, MCFSettingsDialog *Dialog);
    ~MonteCarloSim();
	
protected:
	void closeEvent(QCloseEvent *Event);
	void timerEvent(QTimerEvent *Event);
	
private slots:
	void ProcessFinished(int N, int LP = -1, bool RecursiveCalls = true, FitResult *fitresult = 0);
	void FitTerminated(int N);
	void NParFitsChanged();
	void newConnection();
	void connected();
	void disconnected();
	void readyRead();
	
	inline void FitFinished(int Thread, double)
	{
		ProcessFinished(Thread, -1);
	}
	
private:
	
	void LoadPotential(Potential *Pot, int ResultNum, int ProcessNum);
	void LoadPotential(Potential *Pot, QString FileName, int ProcessNum);
	void CreatePotential(int N, Potential *Pot = 0);
	
	QFile *LogFile;
	QTextStream *LogStream;
	ProcessView **Processes;
	FitResult **BestResults, *Results;
	Potential **Pots;
	int NResults, Task, RAL, CMin, CMax, BorderReached, MaxResult, MinResult, NIpot, NFitsRunning, MaxParFits;
	int NProcesses, NLocalProcesses, *ClientLocalN, NParFits, NFixedCoefficients, *FixedCoefficients;
	int Method, Parameter, Parameter2, Status, MaxBestResults, NPos1, NPos2, *Pos1, *Pos2, ***Field;
	double Prec, RStart, RStart2, RStop, Step, Step2, Value, **Values, *IPotValue1, *IPotValue2;
	bool NoFit, ***Tried, isClient, *Started, internalFitRoutine, Improvement;
	QMessageBox *MBox;
	QString *InitPots, *ClientIP, NameB;
	FieldWindow *fieldWindow;
	QTcpServer *Server;
	QTcpSocket *Socket;
	QStringList MessagesToSend;
	QTime *Time;
	Potential *StartPot;
	QList<IncludePot> StartPotToInclude;
	QList<IncludePot>::const_iterator StartPotIt;
};

#endif // MONTECARLOSIM_H
