//
// C++ Interface: potential
//
//
// Author: Alexander Stein <AlexanderStein@t-online.de>, (C) 2007 - 2016
//
// Copyright: See README file that comes with this source code
//
//

#ifndef POTENTIAL_H
#define POTENTIAL_H

#include "tools.h"
#include "tablewindow.h"
#include "montecarlosim.h"
#include "fit.h"

#include <QDialog>
#include <QThread>

class ElState;
class IsoTab;
class TermEnergy;
class TableLine;
class TermTable;
class FitData;
class NaturalSpline;
class PotFit;
class AnaPot;
class MTTPot;
class MLRPot;
class TangToenniesPot;
class SplinePot;
struct SplinePoint;

class QListWidget;
class QLineEdit;
class QLabel;
class QPushButton;
class QRadioButton;
class QCheckBox;
class QFile;
class QTextStream;
class QMutex;
class QPlainTextEdit;
class QCloseEvent;
class QProgressBar;


struct ELevel
{
	int J, Iso, v;
	double E, *K;
	bool isGT(ELevel L);
	bool isGT(int Iso, int J, int v);
};

struct WantedCoeffValue
{
	int CoeffNum;
	double Value, Precision, CurValue;
};

struct PotentialData
{
	PotentialData();
	~PotentialData();
	
	int nPotCoeff, nLRCoeff, *pLRCoeff, NAdCorr, TAdCorr, PAdCorr, NSpinRGamma;
	double Ri, Ra, iExp, iOff, iCoeff, b, Rm, Tm, De, *PotCoeff, *LRCoeff, A_ex, beta, gamma, q_e, q_f, *adCorr;
	double RIso1AdCorr, RIso2AdCorr, AdCorr_b, AdCorr_Rm, *SpinRGamma, SpinRR1, SpinRR2;
	bool *CFree, *LRCFree, EIFree, *adCorrFree, shortRVariable;
	double *A, *C, Uinf,  iA, iO, alpha, *LRC, Salpha, Sbeta, *SA, *SLRC, Sinf, B, alpha1;
	int NA, N, NLRC, *PLRC;
	int p, q, NCi, Nb, *pCi, *pLRC, NSA, NSLRC, *pSLRC;
	double *bA, *Ci, Re, Rref, T, Ro, FQS;
	SplinePoint *points;
	PotentialType PotType;
	FitResult fitResult;
};


class PotWorker : public QObject
{
	Q_OBJECT
	
public:
	PotWorker(PotFit *Fit, PotentialType PotType = NoPotential);
	PotWorker(const PotWorker &C);
	~PotWorker();
	void MonteCarloSimIteration(double *MCSTab, double UncFact = 1.0);
    double FitAnaPot(int NumWFPoints, bool robustWeighting = false, bool UseSVD = true, double threshhold = 1e-10, bool useLevenbergMarquardt = true,
					 int MaxIt = -1, bool adjustTEWeightFact = false, double hCi_cP = 0.0, bool FitRestarted = false);
    void FitSplinePot(int NumWFPoints, int MaxIt = 1, double threshFakt = 0.9999);
	void FitSplinePotToFunction(double *E, int NPoints, double MinR, double MaxR);
	double FitTangToenniesPot();
    double FitIsoMass(int Iso, double InitStepSize, double &RIsoMass, int NumWFPoints);
	void SetSplinePar(double *Par, double ****E, double *****K, int *mJ, int **mv, int Mv, int N, 
				      int NL, int **QN, double *Sig, double **EQS, double *EO, double *DE, 
                      double &minR, double &maxR, int NumWFPoints);
	AnaPot *getAnaPotForReading();
	AnaPot *getAnaPotForWriting();
	bool setAdCorr(int NAdCorr, double *adCorr, int TAdCorr, int PAdCorr, double RIso1, double RIso2,
				   double Rm = 0.0, double b = 1e-30, bool *AdCorrFree = 0);
	void getAdCorrForReading(int &NAdCorr, double *&adCorr, int &TAdCorr, int &PAdCorr, double &RIso1, double &RIso2, double &Rm, 
							 double &b, bool **AdCorrFree = 0, bool alreadyLocked = false);
	void getAdCorrForWriting(int &NAdCorr, double *&adCorr, int &TAdCorr, int &PAdCorr, double &RIso1, double &RIso2, double &Rm, 
							 double &b);
	bool setSplinePot(int NSplinePoints, SplinePoint *points, int NLRC, int *pLRC, double *LRC, double iA, double iO, double iExp,
					  double LongRangeF = 0.0);
	void getSplinePotForReading(int &NSplinePoints, SplinePoint *&points, int &NLRC, int *&pLRC, double *&LRC, double &iA, double &iO, 
								double &Exp);
	void getSplinePotForWriting(int &NSplinePoints, SplinePoint *&points, int &NLRC, int *&pLRC, double *&LRC, double &iA, double &iO, 
								double &Exp);
	void getLRCoeffForReading(int &N, int *&pLRC, double *&LRC, double *Ra = 0, bool alreadyLocked = false);
	void getLRCoeffForWriting(int &N, int *&pLRC, double *&LRC, bool **LRCFree = 0, double *Ra = 0);
	SplinePoint *getSplinePoints(int &numPoints);
	void setFitData(int *mJ, int **mv, int Mv, int nEs, int *NEL, TermEnergy **EL, int nGS, int *nGP, int *nGL, int **PS, 
					TableLine **GL, bool*ES, int MaxJ, int Maxv);
	int getRefIso();
	void getBadList(QString **&Data, int &NR, int &NC);
	void getDeviations(int &N, int *&Row, double *&dev, double *&RWErr);
	void getFitResult(double &FQS, double &WeightSum, int &NWeightF, int *&WeightFRow, double *&WeightF, int &NumFreePar);
	void getSplineMinMaxR(double &Min, double &Max);
    void showBadList(QString **&Data, int &NR, int &NC, int NumWFPoints);
	void badListChanged(QString *Items, int N, TableLine &L);
	void getSpinRGamma(int &rNSpinRGamma, double *&rSpinRGamma, double &rSpinRR1, double &rSpinRR2, double &rSpinRb, double &rSpinRRm);
	void setSpinRGamma(int NSpinRGamma, double *SpinRGamma, double SpinRR1, double SpinRR2, double SpinRb, double SpinRRm);
	void estMinErr(double ****&MinErr, int NFC, int *mJ, int **mv, int nEs, int *NEL, TermEnergy **EL, 
				   int nGS, int *nGL, TableLine **GL);
	void CreateEK(double ****&E, double *****&K, int *mJ, int **mv, int N);
	void CreateFitEQS(int &N, double **&EQS, double *&EO, double *&DE, double *&Sig, double *&Err, int **&QN, int NP, int nEs, 
					  int *NEL, TermEnergy **EL, int nGS, int *nGP, int *nGL, int **PS, TableLine **GL, int AdRows = 0);
    void calcE(double ****E, int *mJ, int **mv, int Mv, int NumWFPoints, double ****MinErr = 0, bool ****SFQSU = 0, double SFQSRad = 0.0);
	double *getPoints(double Rmin, double Rmax, int numPoints, int FC = 0);
	double getPoint(double R, int FC = 0);
    void getDiagFuncs(double *&WFS, double *&WWFS, int NumWFPoints);
	bool setMolData(IsoTab *IsoT, double *MU, double *IsoF);
	void takeMolData(IsoTab *&IsoT, double *&MU, double *&IsoF);
	void fastWaveFunc(double *U, double E, double M, double h, int NP, double *WF, double &Ri, 
					  double &Ra, double &ISq, double &F, int &i, int &c, int &a);
	double calcFQS(double ****E, int nES, int *NEL, TermEnergy **EL, int nGS, int *nGP, int *nGL, 
                   int **PS, TableLine **GL, int NumWFPoints, bool calcDiag = false, int MJ = 0, int Mv = 0, double *****K = 0,
	   			   int *rv = 0, double *Sig = 0, double TermEnergieF = 1.0, double hCi_cP = 0.0, bool ****SFQSU = 0);
    double getFQS(int NumWFPoints, bool calcDiagFuncs = false, double *StdDev = 0, bool Thread = false, double SFQSRad = 0.0,
				  double *sigma = 0, double *FQS_PAL = 0);
	double *getMCSE(int &N, int *&Rows);
	void unlock();
	void unlockBadList();
	virtual void cdConnectLR(int firstC);
	virtual void cdConnectLR1C();
	virtual void cdConnectSR();
	void getS(double &Rmin, double &Rmax, int &NPoints, int &NC, double **&S);
	double getMinR(double E, double Prec);
	void getMaximum(double &R, double &U);
	virtual void getMinimum(double &R, double &U);
	bool addPoint(double x, double y);
	bool movePoint(int &n, double newx, double newy);
	bool removePoint(int n);
	bool isFitStopped();
	PotWorker* getCopy();
	bool setOmega(int nOmega, double S);
	bool setUinf(double nUinf);
	virtual PotWorker* scalePotential(double newRe, double newDe);
	virtual void shift(double E);
	void getSFQSU(bool ****&SFQSU, int *&mJ, int **&mv);
	void CalcAdCorrPot(double* InPot, double* OutPot, double Rmin, double Step, int NPoints, double MIso1, double MIso2);
	virtual PotentialData *getPotentialData();
	void calcyss();
	virtual void setLRCoeff(double R, int numCoefficients, int *Exponents, double *Coefficients, bool *LRCFree = 0);
	
	inline void setSFQSU(bool ****nSFQSU)
	{
		SFQSU = nSFQSU;
	}
	
	inline FitResult getFitResult()
	{
		return Result;
	}
	
	inline bool isAdCorrA()
	{
		if (NAdCorr > 0) return true;
		return false;
	}
	
	inline bool isLambdaDoublingA()
	{
		return q_e - q_f != 0.0;
	}
	
	inline void getLambdaDoubling(double &qe, double &qf)
	{
		qe = q_e;
		qf = q_f;
	}
	
	inline void setLambdaDoubling(double qe, double qf)
	{
		q_e = qe;
		q_f = qf;
	}
	
	inline int getNSpinRGamma()
	{
		return NSpinRGamma;
	}
	
	inline int getNumFreePar()
	{
		return NumFreePar;
	}
	
	inline int getNIso()
	{
		return NIso;
	}
	
	inline double* getIsoF_ForReading()
	{
		return IsoF;
	}
	
	virtual inline double getUinf()
	{
		return Uinf;
	}
	
	inline IsoTab *getIsoT()
	{
		return IsoT;
	}
	
	inline bool isFitResultAv()
	{
		return dev != 0;
	}
	
	inline bool isFitDataAv()
	{
		return NEL != 0 || nGL != 0;
	}
	
	inline bool isAnaPotAv()
	{
		return Type == analyticalPotential;
	}
	
	inline bool isSplinePot()
	{
		return Type == SplinePotential;
	}
	
	inline bool isLPot()
	{
		return Type == MorseLongRangePotential;
	}
	
	inline bool isMTTPot()
	{
		return Type == ModifiedTangToenniesPotential;
	}
	
	inline void setCalcDiagFuncs(bool calc)
	{
		calcDiagFuncs = calc;
	}
	
	inline bool getCalcDiagFuncs()
	{
		return calcDiagFuncs;
	}
	
	inline void getmvmJ(int *&rmJ, int **&rmv)
	{
		rmJ = mJ;
		rmv = mv;
	}
	
	inline void getSplinePotPointandCoeffNumbers(int &NPoints, int &nLRC)
	{
		NPoints = numSplinePoints;
		nLRC = NLRC;
	}
	
	virtual inline double getIExp()
	{
		return iExp;
	}
	
	inline int *getSplinePLRC()
	{
		return PLRC;
	}
	
	inline bool isTTPot()
	{
		return Type == TangToenniesPotential;
	}
	
	virtual inline bool isExcIntAvailable()
	{
		return false;
	}
	
	inline bool getShortRVariable()
	{
		return shortRVariable;
	}
	
	inline void setLRCFree(bool *nLRCFree)
	{
		if (LRCfree != 0) delete[] LRCfree;
		LRCfree = nLRCFree;
	}
	
	inline void DestroySMap()
	{
		if (SMap != 0) delete[] SMap;
		SMap = 0;
	}
	
	inline void recalcSCLC()
	{
		reCalcLC = true;
		reCalcSC = true;
	}
	
	inline bool isLevenbergMarquardtRunning()
	{
		return LevenbergMarquardtRunning;
	}
	
	inline void setFit(PotFit *nFit)
	{
		Fit = nFit;
	}
	
	virtual inline bool *getLRCFree()
	{
		return LRCfree;
	}
	
	virtual inline bool isFSA()
	{
		return NSpinRGamma > 0;
	}
	
	inline PotentialType getType()
	{
		return Type;
	}
	
	inline double getSFQS()
	{
		return SFQS;
	}
	
	inline void setWantedValues(int numWantedValues, WantedCoeffValue *Values)
	{
		if (NumWantedValues > 0) delete[] WantedValues;
		WantedValues = Values;
		NumWantedValues = numWantedValues;
	}
	
	virtual inline void setWantedLRValues(int numValues, WantedCoeffValue *Values)
	{
		int n, p = numSplinePoints + 1;
		for (n=0; n < numValues; n++) Values[n].CoeffNum += p;
		setWantedValues(numValues, Values);
	}
	
	inline WantedCoeffValue *getWantedValues()
	{
		return WantedValues;
	}
	
	inline void CRotPot(int J, int Iso, double *U, double *R, double minR, double h, int n, int c=0, int FC = 0)
	{
		cRotPot(J, Iso, U, R, minR, h, n, Omega, q_e, q_f, IsoF, SpinRRm, SpinRb, NSpinRGamma, SpinRGamma, 
				SpinRR1, SpinRR2, c, FC);
	}
	
	inline void setR_o(double R_o)
	{
		Ro = R_o;
	}
	
public slots:
	
	void stopFit(bool Stop);
	
signals:
	void potentialImproved(PotentialData *Data);
	void potentialChanged();
	void badListChanged(double aFQS, double aLambda, double mLSFQS);
	void fitStopped();
	void fitRunning();
	void firstFCFCalculated(FitResult fitResult, double FQS);

protected:
	virtual void createSMap();
	virtual void updatePotential(double *C);
	virtual void restorePotential(double *bC);
	virtual void saveCoefficients(double *&bC);
	virtual int cAdRows();
	virtual void getLRCoeff(double &R, int &numCoefficients, int *&Exponents, double *&Coefficients);
	virtual void calc_hCi_cP_EQS(int NEQ, double **EQS, double *Sig, double *EDiff, double hCi_cP);
	virtual void calcS(int numPoints, double RMin, double RMax);
	virtual double Point(double R, int FC = 0);
	virtual void setCurValuesForWantedValues();
	virtual double *Points(double Rmin, double Rmax, int numPoints, int FC = 0);
	
	PotentialType Type;
	SplinePoint *points;
	int numSplinePoints; 
	QMutex *Lock, *BLLock;
	int *SMap;
	int TAdCorr, NAdCorr, PAdCorr, NumFreePar, NumFreeLRC, NumFreeAdCorr, NSpinRGamma, NFitCoeff;
	double *adCorr, RIso1AdCorr, RIso2AdCorr, AdCorr_b, AdCorr_Rm, *AdCorrWF, *SpinRGamma, SpinRR1, SpinRR2, SpinRb, SpinRRm;
	PotFit *Fit;
	bool *adCorrFree, ShowBadList, shortRVariable, *LRCfree, calcDiagFuncs;
	int Omega, *mJ, **mv, Mv, nEs, *NEL, nGS, *nGP, *nGL, **PS, Maxv, MaxJ, NumS, NIso, BadListNR, BadListNC;
	QString **BadListData;
	TermEnergy **EL;
	TableLine **GL;
	double *MU, el_S, **S, SRMin, SRMax, *IsoF, q_e, q_f, *WeightF, FQS, WeightSum, minLSFQS;
	int NLRC, *PLRC, Ndev, *Row, NWeightF, *WeightFRow;
	double **L, *LRC, *dev, *RWErr;
	double Uinf;
	double iO, iA, iExp, Ro;
	IsoTab *IsoT;
	double *WFS, *WWFS, lambda;
	bool reCalcSC, reCalcLC, *ES, StopFit, LevenbergMarquardtRunning, FitStopped;
	FitResult Result;
	
private:
	void destroyFitData();
	void DestroyFitData(double ****E, double *****K, int *mJ, int **mv, int NL,
	   					double **EQS, double *EO, double *DE, double *Sig, double *Err, int **QN);
	void DestroyPotentials();
	void DestroyLS();
	void calcLMatrix();
	void calcLMEQS(int NEQ, double **EQS, double *Sig, double *Diff, double **LMEQS, double hCi_cP);
	int splineFit(bool *RPoints, double &FQSv, double &FQSn, int *&mJ, int **&mv, int &Mv, int &nEs,
				  int *&NEL, TermEnergy **&EL, int &nGS, int *&nGP, int *&nGL, int **&PS, 
                  TableLine **&GL, double &minR, double &maxR, int NumWFPoints, double ****MaxErr = 0, int MaxIt = 1, double threshFakt = 0.9999);
	double TTFit(double ****E, double *****K, int NEQ, int **QN, double *Sig, double **EQS, double *EObs,
				 double *EDiff, double ****MaxErr, double *C, double &FQSv, double b = 0.0);
    double IsoMFit(double ****E, double ****MaxErr, double &FQSv, int Iso, double UFakt, double Mass, int NumWFPoints);
	void fillWantedValueEQS(int NEQ, double *Sig, double **EQS, double *DE);
	bool isStopFit();
	void calcFitResult();
	
	WantedCoeffValue *WantedValues;
	int NumWantedValues;
	double SFQS;
	bool ****SFQSU;
};

class SplinePot : public PotWorker
{
public:
	SplinePot(PotFit *Fit);
	SplinePot(const SplinePot &C);
};

class TangToenniesPot : public PotWorker
{
public:
	TangToenniesPot(PotFit *Fit);
	TangToenniesPot(const TangToenniesPot &C);
	~TangToenniesPot();
	void getMinimum(double &R, double &T);
	void setPotCoeff(double *A, int NA, double b, double *C, int N, double Uinf);
	void getPotCoeff(double *&A, int &NA, double &b, double *&C, int &N, double &Uinf);
	bool calcPotCoeff(double *CC, int NCC, int FNC, double Re, double De);
	double calcPotCoeff(int numPoints, double *R, double *U, double *Sig, int numCoeff, double b);
	double improvePotential(int numPoints, double *R, double *U, double *Sig, double nb = 0.0);
	void updateCoefficients(double *CD);
	PotentialData *getPotentialData();
	
	inline double GetUinf()
	{
		return Uinf;
	}
	
	inline double getb()
	{
		return b;
	}
	
	inline void setb(double nb)
	{
		b = nb;
	}
	
	inline int getNFitCoefficients()
	{
		return NC + NA + (A_ex == 0.0 ? 0 : 1);
	}
	
	inline void setExcInt(double A, double Beta, double Gamma)
	{
		A_ex = A;
		beta = Beta;
		gamma = Gamma;
	}
	
	inline void getExcInt(double &A, double &Beta, double &Gamma)
	{
		A = A_ex;
		Beta = beta;
		Gamma = gamma;
	}
	
	inline void setNA(int nA)
	{
		int n;
		if (A != 0) delete[] A;
		if (nA > 0) for (n=0, A = new double[NA = nA]; n < nA; n++) A[n] = 0.0;
		else A=0;
	}
	
protected:
	void getLRCoeff(double &R, int &numCoefficients, int *&Exponents, double *&Coefficients);
	void calcS(int numPoints, double Rmin, double Rmax);
	double Point(double R, int FC = 0);
	
private:
	void getPreFakt(double R, double *C);
	bool calcAb(double Re, double De);
	void AB(double BI, double &Y, double &AA, double *D, double RM);
	
	double Uinf, *A, b, *C, *SWF, A_ex, beta, gamma, RMin, UMax;
	int NC, NA;
};

class AnaPot : public PotWorker
{
	public:
		AnaPot(PotFit *Fit);
		AnaPot(const AnaPot &C);
		~AnaPot();
		AnaPot *scalePotential(double newRe, double newDe);
		void setInnerWall(double R, double Offset, double Exponent, double Coefficient);
		void getInnerWall(double &R, double &Offset, double &Exponent, double &Coefficient);
		void setPotCoeff(double b, double Rm, double Tm, double De, int numCoefficients, 
						 double *Coefficients, bool *CoeffFree = 0);
		void getPotCoeff(double &b, double &Rm, double &Tm, double &De, int &numCoefficients,
						 double *&Coefficients);
		void setLRCoeff(double R, int numCoefficients, int *Exponents, double *Coefficients, bool *LRCFree = 0);
		void setExchangeCoeff(double A_ex, double beta, double gamma, bool Free = true);
		void getExchangeCoeff(double &A_ex, double &beta, double &gamma);
		void cdConnectLR1C();
		void cdConnectLR(int firstCoeff);
		void cdConnectSR();
		double calcPotCoeff(int numPoints, double *R, double *U, double *Sig, int numCoeff, double Rm, double b);
		double fitPotential(int NPoints, double R0, double h, double *U, double *Sig, int NCoeff, int NLRC, int *pLRC,
							double Ri, double Ra, double Rm, double b, double Uinf, double iExp);
		void fitAdCorrToRmb();
		void getMinimum(double &R, double &T);
		double getUinf();
		void sConnect();
		void shift(double Energy);
		void getConR(double &Ri, double &Ra);
		void getNC(int &N, int &NC, int &NLRC);
		void getDerivatives(double r, double *dC, double *dLRC);
		PotentialData *getPotentialData();
		
		inline void setDe(double nDe)
		{
			Uinf = nDe;
		}
		
		inline bool *getCoeffFree()
		{
			return CoeffFree;
		}
		
		inline bool *getLRCFree()
		{
			return LRCFree;
		}
		
		inline bool isExcIntFree()
		{
			return ExcIntFree;
		}
		
		inline bool isExcIntAvailable()
		{
			return ExcIntFree || A_ex != 0.0;
		}
		
		inline void getRmb(double &rRm, double &rb)
		{
			rRm = Rm;
			rb = b;
		}
		
		inline double getIExp()
		{
			return iExp;
		}
		
	protected:
		double *Points(double Rmin, double Rmax, int nPoints, int FC = 0);
		void saveCoefficients(double *&bC);
		void updatePotential(double *C);
		void getLRCoeff(double &R, int &numCoefficients, int *&Exponents, double *&Coefficients);
		void createSMap();
		void restorePotential(double *bC);
		void calc_hCi_cP_EQS(int NEQ, double **EQS, double *Sig, double *EDiff, double hCi_cP);
		void calcS(int numPoints, double Rmin, double Rmax);
		double Point(double r, int typ = 0);
		int cAdRows();
		
	private:
		double Ri, Ra, iOff, iCoeff, *PotCoeff, *LRCoeff, Tm, *SWF;
		int nLRCoeff, *pLRCoeff;
		int nPotCoeff, numFreeCoeff, numFreeLRC;
		double Rm, b, A_ex, beta, gamma;
		bool *LRCFree, *CoeffFree, ExcIntFree, ADLRC;
};

class MLRPot : public PotWorker
{
	public:
		MLRPot(PotFit *Fit);
		MLRPot(const MLRPot &C);
		~MLRPot();
		MLRPot *scalePotential(double newRe, double newDe);
		void getMinimum(double &R, double &T);
		void getDerivatives(double R, double &E, double *dbeta, double *dC, double &dRe, double &dDe);
		void getNC(int &N, int &NC, int &NLRC);
		double GetDe();
		double getUinf();
		void shift(double Energy);
		void setCoefficients(int p, int q, double TAs, double De, double Re, double Rref, 
							 int Nbeta, double *beta, int NLRC, int *pLRC, double *LRC);
		void getCoefficients(int &p, int &q, double &TAs, double &De, double &Re, double &Rref,
							 int &Nbeta, double *&beta, int &NLRC, int *&pLRC, double *&LRC);
		void calcRIndP();
		void test();
		PotentialData *getPotentialData();
		
	protected:
		void getLRCoeff(double &R, int &numCoefficients, int *&Exponents, double *&Coefficients);
		double Point(double r, int FC = 0);
		
	private:
		int Nbeta, NLRC, p, q, *pLRC;
		double Re, Rep, Rref, Rrefq, Rrefp, De, *Ci, *beta, betainf, URe, TAs, *dCiLR, dULRdRe;
};

class MTTPot : public PotWorker
{
public:
	MTTPot(PotFit *Fit);
	MTTPot(const MTTPot &C);
	~MTTPot();
	void getMinimum(double &R, double &T, int c=0);
	double GetDe(int c=0);
	double GetUinf(int c=0);
	MTTPot *scalePotential(double newRe, double newDe);
	void setCoefficients(double B, double alpha1, int NA, double *A, double alpha, double beta, double gamma, 
						 int NLRC, int *pLRC, double *LRC, double Uinf, int NC = 0,
						 double *C = 0, double Calpha = 0.0, double Cbeta = 0.0,
						 int NCLRC = 0, int *pCLRC = 0, double *CLRC = 0, 
					     double CInf = 0.0);
	void getCoefficients(double &B, double &alpha1, int &NA, double *&A, double &alpha, double &beta, double &gamma,
						 int &NLRC, int *&pLRC, double *&LRC, double &Uinf, int &NC,
						 double *&C, double &Calpha, double &Cbeta, int &NCLRC,
					     int *&pCLRC, double *&CLRC, double &CInf);
	PotentialData *getPotentialData();
	
	inline bool isFSA()
	{
		if (NC > 0 && Calpha != 0.0 && Cbeta != 0.0 && NCLRC > 0) return true;
		return false;
	}
	
protected:
	void getLRCoeff(double &R, int &numCoefficients, int *&Exponents, double *&Coefficients);
	double Point(double r, int c=0);
	
private:
	int NA, NLRC, *pLRC, NC, NCLRC, *pCLRC;
	double *A, alpha, beta, gamma, *LRC, Uinf, *C, Calpha, Cbeta, *CLRC, CInf, B, alpha1;
};

class CMLRPotDialog : public QDialog
{
	Q_OBJECT
	
public:
    CMLRPotDialog(QStringList *CVL, QWidget* parent = 0);
	
	QLineEdit *pE, *qE, *RrefE, *mRE, *MRE, *NPE, *NCE;
	QListWidget *LRC;
	
private slots:
	void addC();
	void delC();
	void setV();
	
private:
	QComboBox *LRCB;
	QPushButton *addB, *delB, *setB, *OK, *Cancel;
	QStringList *CVL;
	QLineEdit *CValE;
};

class CoefficientDialog : public QDialog
{
	Q_OBJECT
	
	public:
		CoefficientDialog(QWidget *parent, QStringList &Coefficients);
		QStringList getResults();
		
	private slots:
		void add();
		void remove();
		
	private:
		QListWidget *CList;
		QLineEdit *Edit;
		QLabel *Label;
		QPushButton *Add, *Remove, *Cancel, *OK;
};

class CoupledSineWaveFunc : public MDIChild
{
public:
    CoupledSineWaveFunc(MainWindow *MW, Molecule *Mol);
    ~CoupledSineWaveFunc();
	bool readData(QString FN = "");
	bool writeData(QString FN = "");
	int getNChannels();
	int getNLevels(int I, int J);
	void getStateComp(int Channel, ElState *&State, int &Comp);
	bool getWaveFunc(int I, int J, int v, int Channel, double RStart, double RStop, int NPoints, double *Data, double &E, 
					 int &MMC);
	bool getWaveFuncs(int I, int J, int Channel, double RStart, double RStop, int NPoints, double **WF, double *E);
	bool getWaveFuncs(int I, int v, int *Channels, int NChannels, int NJ, int *J, double RStart, 
					  double RStop, int NPoints, double ***WF, double *E, int *vAssignment,
					  int *ChannelAssignment);
	bool getWaveFuncs(int I, int J, double RStart, double RStop, int NPoints, double ***WF, double *E);
	void getv(int I, int J, int &N, int *&v);
	int getv(int I, int J, double E);
	bool isJA(int I, int J);
	void getIso(int &N, int *&I);
	void setData(int NI, int NJ, int Nv, int NChan, int NCoeff, ElState **St, int *C, int *I, double *R, double *****Data,
				 double ****MC, double ***E);
	double ****getEnergy();
	
	inline int getNJ()
	{
		return nJ;
	}
	
	inline int getNv()
	{
		return nv;
	}
	
	inline QString getSource()
	{
		return Source;
	}
	
	inline void setSource(QString S)
	{
		Source = S;
	}
	
	inline void Assign()
	{
		ASC++;
	}
	
	inline bool Delete()
	{
		if (--ASC == 0) return true;
		return false;
	}
	
	inline int getIsoT(int I)
	{
		int n;
		for (n=0; n < nI; n++) if (I == Iso[n]) return n;
		return -1;
	}
	
private:
	void DestroyData();
	void CalcBasis(double rMin, double rMax, int NPoints);
	void CalcvCA();
	
	int nI, nJ, nv, nChan, nCoeff, *Iso, *SC, ASC, nPoints, ***vA, ***CA;
	double *R, *****Data, ****MC, ***E, **Basis, RMin, RMax;
	ElState **States;
	Molecule *Mol;
	QString Source;
	NaturalSpline *MapFuncs;
};

class CSWFImportDialog : public QDialog
{
	Q_OBJECT
	
public:
    CSWFImportDialog(MainWindow *MW, QString InitDir);
	void getData(Molecule *&Mol, ElState **&States, int *&Components, int &NStates, int *&Iso, 
				 QString *&IsoDirs, int &NIso, QString &PotentialFile, QString &T5_E_BvFile);

private slots:
	void molChanged(int i);
	void addState();
	void addIso();
	void removeState();
	void removeIso();
	void selectPotFile();
	void selectT5_E_Bv();
	void selectIsoDir();
	
private:
	
	int *ST, *CT;
	MainWindow *mw;
	Molecule *mol;
	QListWidget *SList, *IsoList;
	QComboBox *SBox, *MolBox, *IsoBox;
	QPushButton *AddState, *RemoveState, *AddIso, *RemoveIso, *SelectPotFile, *SelectT5_E_Bv, *SelectIsoDir, *OK, *Cancel;
	QLineEdit *PotFile, *T5_E_Bv, *IsoDir;
	QList<int> IL1, IL2, SL1, SL2;
};

class MonteCarloSimDialog : public QDialog
{
	Q_OBJECT
	
public:
    MonteCarloSimDialog(MainWindow* parent);
	bool exec(QString &Directory, int &NIterations, int &NParFit, double &UncFact);
	
private slots:
	void showFileDialog();
	
private:
	QLineEdit *DirectoryE, *NItE, *NParFitsE, *UncFactE;
};

class CreateAnaPotSeriesFromMCSSplinePotSeriesDialog : public QDialog
{
	Q_OBJECT
public:
    CreateAnaPotSeriesFromMCSSplinePotSeriesDialog(MainWindow* parent);
	void getData(QString &MolFN, QString &StateN, QString &FDDir, QString &SPDir, QString &APDir, 
				 bool &improveAnaPots, bool &UseSvd, bool &UseLeveMarq, int &MaxIt, double &Prec);

private slots:
	void showFDDirDialog();
	void showSplinePotDirDialog();
	void showAnaPotDirDialog();
	void molChanged(int i);
	void improveChanged(bool ckecked);
	void MaxItChanged(const QString &text);
	
private:
	MainWindow *MW;
	ElState *State;
	QLineEdit *FDDirE, *SplinePotDirE, *AnaPotDirE, *PrecE;
	QComboBox *MoleculeBox, *StateBox, *MaxIt;
	QCheckBox *improve, *useSVD, *useLM;
	QPushButton *SPDB;
};

class FitAnaPotDialog : public QDialog
{
	Q_OBJECT
public:
    FitAnaPotDialog(bool improve, double Ri, double Ra, double iExp, int NC, int NLRC, int *pLRC, double hCi_cP, 
					PotentialType CurrentPotential, QWidget* parent = 0);
	bool exec(bool &improve, bool &useLeveMrq, bool &useSVD, double &Ri, double &Ra, double &iExp, int &NC, int &NLRC, int *&pLRC, 
			  double &Rmin, double &Rmax, int &NumP, double &bmin, double &bmax, double &bstep, bool &adjustTEWeightFact, 
		      double &hCi_cP, PotentialType &NewType);
	
private slots:
	void improveChanged(bool State);
	void addCoeff();
	void removeCoeff();
	void typeChanged(int Index);
	
private:
	QListWidget *CoeffList;
	QComboBox *CoeffBox, *TypeBox;
	QPushButton *Add, *Remove;
	QLineEdit *RiE, *RaE, *iExpE, *NCE, *RminE, *RmaxE, *NumPE, *bminE, *bmaxE, *bstepE, *hCi_cPE;
	QCheckBox *Improve, *UseLeveMrq, *UseSVD, *AdjustTEWF;
	PotentialType CurPotential;
};

class PotFitLogWindow : public MDIChild
{
	Q_OBJECT
	
public:
	PotFitLogWindow(QWidget *parent = 0);
	void addTextRow(QString Text);
	bool askForQuit();
	
signals:
	void CancelFit(bool wait);
	void StopFit();
	void StartFit();
	void RestartFit();
	
public slots:
	void FitStopped();
	void FitFinished();
	void FitTerminated();
	void FitRunning();
	
private slots:
	void cancelFit();
	void stopFit();

private:
	QPlainTextEdit *Log;
	QPushButton *Stop, *Cancel, *Close;
	bool fitFinished, fitStopped, fitRunning;
};

class PotFit : public QThread
{
	Q_OBJECT
	
public:
	enum Task {PotentialFit, MonteCarloSimulation, calcFQS};
	
    PotFit();
	void FitAnaPot(bool robustWeighting = false, bool UseSVD = true, double threshhold = 1e-10,
				   bool useLevenbergMarquardt = true, int MaxIt = -1, bool adjustTEWeightFact = false, double hCi_cP = 0.0);
	void setWorker(PotWorker *Worker);
	
	inline void FitSplinePot(int nMaxIt = 1, double threshFakt = 0.9999)
	{
		ToDo = PotentialFit;
		FitPot = SplinePotential;
		MaxIt = nMaxIt;
		threshhold = threshFakt;
		start();
	}
	
	inline void CalcFQS(double nSFQSRad = 0.0)
	{
		ToDo = calcFQS;
		SFQSRad = nSFQSRad;
		start();
	}
	
	inline void restartFit()
	{
		restart = true;
		start();
	}
	
	inline void RunMonteCarloIteration(double *MCSTab, double UncFact = 1.0)
	{
		ToDo = MonteCarloSimulation;
		MCST = MCSTab;
		MCSUncFact = UncFact;
		start();
	}
	
	inline PotFit::Task getTask()
	{
		return ToDo;
	}
	
protected:
	void run();
	
private:
	
	Task ToDo;
	bool robustWeighting, UseSVD, adjustTEWeightFact, useLevenbergMarquardt, restart;
	int MaxIt;
	double threshhold, hCi_cP, *MCST, SFQSRad, MCSUncFact;
	PotWorker *Worker;
	PotentialType FitPot;
};

class Potential : public TableWindow
{
	Q_OBJECT
			
public:
    Potential(MainWindow *MW = 0, Molecule *Mol = 0, int ThreadNum = -1);
	Potential(const Potential &C);
    ~Potential();
	Potential *scalePotential(double newRe, double newDe);
    void getEV(double ****&Ev, int &Nc, int &NI, int &Nv, int &NJ, int NumWFPoints);
    void exportAsymptoticLevels(QString FileName, int numv, int maxJ, int NumWFPoints);
	bool readPotData(QString Filename = "");
	bool readCoupledPotData(QString Filename = "");
	bool writePotData(QString Filename = "");
	bool readPointPot(QString Filename = "");
    void calcTermEnergies(TermTable *&TT, int NumWFPoints, int Mv = -1, int MJ = -1, bool show = true);
	void getWaveFunc(int I, int J, int &Nv, double *&E, double **&WF, int *&ri, int *&ra, int *&P, 
                     double *&F, double *&N, int NumWFPoints);
	void getFCF(int &Nv1, int &Nv2, double *E1, double *E2, double **WF1, double **WF2, int *ri1,
				int *ri2, int *ra1, int *ra2, int *P1, int *P2, double *F1, double *F2,
                double *N1, double *N2, double ***&FCF, double Min1 = 0.0, double Min2 = 0.0);
    void getFCF(Potential *uPot, int Iso, int Js, int Jss, int &Nvs, int &Nvss, double ***&FCF, int NumWFPoints);
	void getFastWaveFunc(int I, int J, int C, double E, double M, double h, int NP, double *WF, double &Ri, 
						 double &Ra, double &ISq, double &F, int &i, int &c, int &a);
	void getFastFCF(int I, Potential *uPot, int Js, int uC, double uE, int N, double *lE, int *lJss, 
                    double *FCF, int NumWFPoints, double **uWF = 0, double *TStr = 0, int NWFC = 0);
	double calcScatWaveFunc(bool write = true, double **nodePos = 0, double maxR = 5000.0);
	double **fitNodePos();
	bool readData(QString Filename = "");
	bool writeData(QString Filename, bool writeFitData);
	void writePoints(QString FileName, double RMin, double RMax, int NPoints, int NDigits, bool FirstRow, bool AdCorrE, int Iso, 
					 QString MCSDir);
	void setPointData(int nPoints, double *R, double *E);
	void setMTTCoefficients(double B, double alpha1, int NA, double *A, double alpha, double beta, double gamma, 
							int NLRC, int *pLRC, double *LRC, double Uinf, int NC = 0,
							double *C = 0, double Calpha = 0.0, double Cbeta = 0.0,
							int NCLRC = 0, int *pCLRC = 0, double *CLRC = 0, 
							double CInf = 0.0);
	bool setElState(ElState *nState);
	ElState *getElState();
	void setMolecule(Molecule *Mol);
    void getTexTable(int NumWFPoints, double FQS = -1.0);
	void UpdatePot(int type = -1);
	double getMinR();
	double getMaxR();
    double getD0(int NumWFPoints);
	int getNFC();
	void getReDe(double &Re, double &De);
	double getMinimumE();
	double getAsymptote();
	double *getAdCorrEnergy(double Rmin, double Step, int numPoints, double mIso1, double mIso2);
    double getFQS(int NumWFPoints, bool calcDiagFuncs = false, double *StdDev = 0, double *sigma = 0, double *FQS_PAL = 0);
	void FitSplinePot(bool showResult = true, int MaxIt = 1, double threshFakt = 0.9999);
	double CreateSplineFromAnalyticalPot();
	void FitAnaPot(bool showResult = true, bool robustWeighting = false, bool UseSVD = true, double threshhold = 1e-10,
				   bool transform = false, bool ask = true, bool useLevenbergMarquardt = true, int MaxIt = -1,
				   bool adjustTEWeightFact = false, double hCi_cP = 0.0, bool WritePotFitTraceTab = false);
	double FitTangToenniesPot(bool showResult = true);
    double FitIsoMass(int Iso, double InitStepSize, double &RIsoMass, int NumWFPoints);
	void MonteCarloSimIteration(double *MCSTab, double UncFact = 1.0);
	void VaryCoefficients(bool vary, int N=0, int *Rows = 0);
	void getFixedCoefficients(int &N, int *&Rows);
	void setMinimum(double Energy);
	void setAsymptote(double Energy);
	void shiftEnergyOffset(double Energy);
	void cdConnectSR();
	void cdConnectLR(int firstC);
    void showBadList(int NumWFPoints);
    void autoCalcScatLengthsPotentialSet(int NumWFPoints);
	void setCoefficients();
	void addPoint(double x, double y);
	void movePoint(int &n, double newx, double newy);
	void removePoint(int n);
	void setCalcDiagFuncs(bool calc);
	bool getCalcDiagFuncs();
    void getDiagFuncs(double *&WaveFSum, double *&WWFS, double &RStart, double &h, int &NP, int NumWFPoints);
	FitData *getFitData();
	void showFitData();
	void updateFitData();
	void createMLRPot();
	void setCouplingData(CoupledSineWaveFunc *WF, int NCoupled, Potential **CoupledPot, int *CoupledComp);
	void setCouplingData(CoupledSineWaveFunc *WF, int NCoupled, ElState **CoupledStates, int *CoupledComp);
	CoupledSineWaveFunc *getCoupledSineWaveFunc();
	void getCoupledPotentials(int &nCoupled, Potential **&potentials, int *&components);
    void exportWaveFunction(int NumWFPoints);
	void getLRCoeffForReading(int &N, int *&pLRC, double *&LRC);
	void getLRCoeffForWriting(int &N, int *&pLRC, double *&LRC, bool **LRCFree = 0);
	void getExchangeInt(double &A, double &alpha, double &gamma);
	void setExchangeInt(double A, double alpha, double gamma);
	bool setAdCorr(int NAdCorr, double *adCorr, int TAdCorr, int PAdCorr, double RIso1, double RIso2,
				   double Rm = 0.0, double b = 1e-30);
	void getAdCorrForWriting(int &NAdCorr, double *&adCorr, int &TAdCorr, int &PAdCorr, double &RIso1, double &RIso2, double &Rm, 
							 double &b);
	FitResult getFitResult();
	PotentialType getPotType();
	void cutRows(int &numRows, int &numColumns, QString **&Data);
	void insertRows(int numRows, int numColumns, QString **Data);
	void DeleteRows();
	void setWorker(PotWorker *NWorker);
	void calcFQS_SFQS(bool ****SFQSU = 0, double SFQSRad = 0.0);
	void calcyss();
	
	inline void getSFQSU(bool ****&SFQSU, int *&mJ, int **&mv)
	{
		Worker->getSFQSU(SFQSU, mJ, mv);
	}
	
	inline double *getPoints(double Rmin, double Rmax, int numPoints, int FC = 0)
	{
		return Worker->getPoints(Rmin, Rmax, numPoints, FC);
	}
	
	inline double getPoint(double R, int FC = 0)
	{
		return Worker->getPoint(R, FC);
	}
	
	inline bool isAdCorrA()
	{
		return Worker->isAdCorrA();
	}
	
	inline int getNCoupledPotentials()
	{
		return NCoupled;
	}
	
	inline void setData(QString **Data, int NRows, int NCols)
	{
		TableWindow::setData(Data, NRows, NCols);
		updatePot();
	}
	
	inline bool writeData(QString Filename = "")
	{
		return writeData(Filename, true);
	}
	
	inline IsoTab *getIsoT()
	{
		return Worker->getIsoT();
	}
	
	inline void getAdCorrForReading(int &NAdCorr, double *&adCorr, int &TAdCorr, int &PAdCorr, double &RIso1, double &RIso2, double &Rm, 
							 double &b)
	{
		Worker->getAdCorrForReading(NAdCorr, adCorr, TAdCorr, PAdCorr, RIso1, RIso2, Rm, b);
	}
	
	inline void SetSplinePar(double *Par, double ****E, double *****K, int *mJ, int **mv, int Mv, int N, 
							 int NL, int **QN, double *Sig, double **EQS, double *EO, double *DE, 
                             double &minR, double &maxR, int NumWFPoints)
	{
        Worker->SetSplinePar(Par, E, K, mJ, mv, Mv, N, NL, QN, Sig, EQS, EO, DE, minR, maxR, NumWFPoints);
	}
	
	inline SplinePoint *getSplinePoints(int &numPoints)
	{
		return Worker->getSplinePoints(numPoints);
	}
	
	inline void getS(double &Rmin, double &Rmax, int &NPoints, int &NC, double **&S)
	{
		Worker->getS(Rmin, Rmax, NPoints, NC, S);
	}
	
	inline double getMinR(double E, double Prec)
	{
		return Worker->getMinR(E, Prec);
	}
	
	inline void getMaximum(double &R, double &U)
	{
		Worker->getMaximum(R, U);
	}
	
	virtual inline void getMinimum(double &R, double &U)
	{
		Worker->getMinimum(R, U);
	}
	
	inline void CRotPot(int J, int Iso, double *U, double *R, double minR, double h, int n, int c=0, int FC = 0)
	{
		Worker->CRotPot(J, Iso, U, R, minR, h, n, c, FC);
	}
	
	inline void CalcFQS()
	{
		if (molecule == 0 || Fit->isRunning()) return;
		FQSv = -1.0;
		bFQS = 1e300;
		setFitData();
		Fit->CalcFQS();
	}
	
	inline void setThreadNum(int N)
	{
		threadNum = N;
	}
	
	inline bool isExcIntAvailable()
	{
		return Worker->isExcIntAvailable();
	}
	
	inline int getLRCTabOffset()
	{
		return LRCTabOffs;
	}
	
	inline int getadCorrTabOffset()
	{
		return adCorrTabOffs;
	}
	
	inline int getExcIntATabPos()
	{
		return ExcIntATabPos;
	}
	
	inline bool isFitRunning()
	{
		return Fit->isRunning();
	}
	
	inline bool isFitStopped()
	{
		bool R = Worker->isFitStopped();
		Worker->unlock();
		return R && Fit->isFinished();
	}
	
	inline void cdConnectLR1C()
	{
		Worker->cdConnectLR1C();
	}
	
	inline void FitSplinePotToFunction(double *E, int NPoints, double minR, double maxR)
	{
		Worker->FitSplinePotToFunction(E, NPoints, minR, maxR);
	}
	
	inline double getUinf()
	{
		return Worker->getUinf();
	}
	
	inline void setWantedValues(int numWantedValues, WantedCoeffValue* Values)
	{
		Worker->setWantedValues(numWantedValues, Values);
	}
	
	inline void setWantedLRValues(int numWantedValues, WantedCoeffValue *Values)
	{
		Worker->setWantedLRValues(numWantedValues, Values);
	}
	
	inline WantedCoeffValue *getWantedValues()
	{
		return Worker->getWantedValues();
	}
	
signals:
	void FitFinished(int ThreadNum, double FQS);
	void FitTerminated(int ThreadNum);
	void SFQScalculated(int ThreadNum, double SFQS, double FQS);
	
private slots:
	void updatePot();
	void badListChanged(QString *Items, int N);
	void badListChanged(double aFQS, double aLambda, double mLSFQS);
	void potentialImproved(PotentialData *Data);
	void fitFinished();
	void fitTerminated();
	void UpdateTab(PotentialData *Data = 0);
	void CreateFirstPotFitTraceTabRows(FitResult Result, double FQS);
	void restartFit();
	void startFit();
	void cancelFit(bool wait);
	
	inline void stopFit()
	{
		Worker->stopFit(true);
	}
	
private:
	bool getFastWaveFuncs(int C, int I, int J, int Nv, double M, double h, int NP,
						  double **WF, double Ri, double Ra, double *WFsq, double *F,
						  int *i, int *c, int *a);
	void NCNF(double M, double E, double *Y, double *WF, int d);
	double PFakt(double M, double E);
	void calcDerivatives(TableLine *Lines, int NL, double *E, double **dyda);
	void setFitData(int maxv = 0, int maxJ = 0, int Iso = -1, bool OnlyTermEnergies = false);
	void initLogWindow();
	bool testIfSplinePot();
	double *calcSigFuncs(int numPoints, double rMin, double rMax);
						
	int fitN_C, fitN_I, fitN_N, fitN_MI, LRCTabOffs, adCorrTabOffs, ExcIntATabPos;
	double fitN_V, fitN_A, fitN_maxr, FQSv, bFQS;
	ElState *State;
	int maxSplinePoints, NFC, threadNum, NFitIt;
	bool showFitResult, getTexTableSplineFitRunning, writePotFitTraceTab;
	TableWindow *BadList;
	QPixmap *FixPix;
	FitData *fitData;
	CoupledSineWaveFunc *WaveFuncs;
	Potential **CoupledPot;
	int *CoupledComp, NCoupled;
	PotFitLogWindow *LogWindow;
	PotWorker *Worker;
	PotFit *Fit;
	QFile *DebugFile;
	QTextStream *DebugStream;
	QStringList PotFitTraceTab;
};

class ThreadControl : public QWidget
{
	Q_OBJECT
	
public:
    ThreadControl(MainWindow* MW, ElState *St, int NumIterations, int NumParFits);
    virtual ~ThreadControl();
	
public slots:
	void CalcTerminated(int ThreadNum);
	
protected:
	virtual void CalcFinished(int ThreadNum);
	void closeEvent(QCloseEvent *event);
	void StartCalcs();
	virtual void StartCalc(int p, int n);
	virtual void AllCalculationsFinished();
	
	int NumParF, MMParF, MaxParF, NumIt, NumTermF, *ItI, *PotI, ItFinished, NFC, *FCRows;
	int NumResultTabColumns, AdTabRows;
	Potential **Pots;
	ElState *St;
	QString **PotData, ResultDir, PotName, PotFileName;
	TableWindow *ResultTab;
	MainWindow *MW;
	FitData *MFD;
	
	QProgressBar *Progress;
	QLineEdit *NumParFE, *NumItE;
	QIntValidator *NumParFEValid, *NumItEValid;
	
	
protected slots:
	void NItChanged();
	void NParFChanged();
};


class MonteCarloSimControl : public ThreadControl
{
	Q_OBJECT
	
public:
    MonteCarloSimControl(MainWindow* MW, Potential *Pot, QString PotDir, int NumIterations, int NumParFits, double UncFact, int NumWFPoints);
    ~MonteCarloSimControl();
	
public slots:
	void FitFinished(int ThreadNum, double FQS);
	
protected:
	void StartCalc(int p, int n);
	
private:
	int NumPDRows, NumPDColumns;
	double *MCSTab, UncF;
	FitData *MCSFD;
};


class SFQSCalcControl : public ThreadControl
{
	Q_OBJECT
	
public:
	SFQSCalcControl(MainWindow *MW, ElState *St, Potential *OPot, QString PotDir, QString FDatDir, double SFQSRad, int NumParFits); 
    virtual ~SFQSCalcControl();
	
public slots:
	void CalcFinished(int ThreadNum, double SFQS, double FQS);
	
protected:
	void StartCalc(int p, int n);
	
private:
	QString PDir, FDir;
	QStringList PotList;
	double Rad;
	bool ****SFQSU;
	int *mJ, **mv;
	Potential *OPot;
};

class SFQSCalcDialog : public QDialog
{
	Q_OBJECT
	
public:
	SFQSCalcDialog(MainWindow *MW, QString Dir);
	void exec(ElState *&St, Potential *&OPot, QString& PotDir, QString &FDatDir, double &SFQSRad, int &NumParFits);
	
private slots:
	void MolChanged(int n);
	void StateChanged(int n);
	void sPotDir();
	void sFDatDir();
	
private:
	QComboBox *MolBox, *StateBox, *PotBox;
	QLineEdit *PotDirE, *FDatDirE, *SFQSRadE, *NumParFitsE;
	QPushButton *sPotDirB, *sFDatDirB, *OK, *Cancel;
	QString Dir;
	MainWindow *MW;
};

class ImprovePotSeriesControl : public ThreadControl
{
	Q_OBJECT
	
public:
	ImprovePotSeriesControl(MainWindow *MW, ElState* St, Potential *OPot, FitData *FDat, QString PotDir, double Threshold, 
							int NumParFits);
	
public slots:
	void FitFinished(int ThreadNum, double FQS);
	
protected:
	void StartCalc(int p, int n);
	void PreparePot(int p, int n);
	virtual QString getWriteFileName(QString CurrentFileName);
	
	QString PotDir;
	QStringList PotList;
	double threshold;
	bool UseLeveMarq;
};

class CreateAnaPotSeriesControl : public ImprovePotSeriesControl
{
public:
    CreateAnaPotSeriesControl(MainWindow* MW, ElState* St, Potential* OPot, FitData* FDat, QString SplinePotDir, QString FitDataDir, QString ResultPotDir, 
							  double Threshold, int NumParFits, int MaxFitIterations, int NStart, bool improveAnaPots, bool UseSvd, bool UseLeveMarq);
    virtual ~CreateAnaPotSeriesControl();
	
protected:
	void CalcFinished(int ThreadNum);
	void StartCalc(int p, int n);
	void AllCalculationsFinished();
	QString getWriteFileName(QString CurrentFileName);
	
private:
	QString FitDataDir, ResultPotDir, MFD;
	int MaxFitIterations, NStart, PotN, FDN;
	QTextStream *LogStream;
	QFile *LogFile;
	bool improveAnaPots, UseSvd, UseLeveMarq;
	FitData *FDat;
};

class ImprovePotSeriesDialog : public QDialog
{
	Q_OBJECT
	
public:
	ImprovePotSeriesDialog(MainWindow *MW, QString Dir, Potential *OPot);
	bool exec(ElState *&St, FitData *&FDat, QString &PotDir, double &Threshold, int &NumParFits);
	
private slots:
	void MolChanged(int i);
	void StateChanged(int i);
	void sPotDir();
	
private:
	MainWindow *mw;
	Molecule *Mol;
	ElState *St;
	QComboBox *MoleculeB, *StateB, *FitDataB;
	QLineEdit *ThresholdE, *NumParFitsE, *PotDirE;
};

Q_DECLARE_METATYPE(FitResult)

#endif
