//
// C++ Interface: residualplot
//
//
// Author: Alexander Stein <AlexanderStein@t-online.de>, (C) 2009 - 2016
//
// Copyright: See README file that comes with this source code
//
//

#ifndef RESIDUALPLOT_H
#define RESIDUALPLOT_H

#define TestResidualFit

#include <DiagWindow.h>
#include "naturalspline.h"

class MainWindow;
class Molecule;
class TermTable;
class LineTable;
class FitData;

class QPaintDevice;
class QPainter;


class ResidualPlot : public DiagWindow
{
	Q_OBJECT
	
	public:
    	ResidualPlot(MainWindow *MW);
		~ResidualPlot();
		void PSpektrum(QPainter &P, const QRect & A, bool PrintFN);
		void ClearMarked();
		
	public slots:
		void molBoxChanged();
		void sourceBoxChanged();
		void refSourceBoxChanged();
		void moleculesChanged();
		
	private slots:
		void stateBoxChanged(int Index);
		void isoBoxChanged(int Index);
		void vBoxChanged(int Index);
		void compBoxChanged(int Index);
		void Printall();
		void markLines();
		void setError();
		void add9toError();
		void sub9fromError();
		void deleteLevels();
		void showPMenu(QPoint *P);
		void PictureClicked(QPoint *P, bool ControlPressed);
		void marked(QRect *R, bool ControlPressed);
		void markUnc();
		void markComp();
		
	private:
		void drawDataPoint(QPainter *P, int x, int y, int unc1, int unc2, int end);
		void destroyData();
		void initData();
		void mark(int xi, int xo, int yi, int yo, bool ControlPressed);
		
		Molecule *mol;
		ElState *state;
		TermTable *refT, *termT;
		LineTable *lineT;
		FitData *FitD;
		QMenu *PMenu;
		QAction *setErrorAct, *add9toErrorAct, *sub9fErrorAct, *deleteLevelsAct, *markLinesAct, *markUncAct, *markCompAct;
		QList<int> CompList;
		double ****Data, ****Unc;
		int ***nData, *nv, ****JData, Nv, NC, NJ, NIso, v, C, Iso, ****Row, ****Comp;
		int MrefTcT, *refTcT, *refTIsoT;
		bool Block, *Marked, FSA;
		
#ifdef TestResidualFit
		ResidualFit TestResFit;
		DiagWindow *SPlot;
#endif
};

#endif
