//
// C++ Interface: termtable
//
//
// Author: Alexander Stein <AlexanderStein@t-online.de>, (C) 2006 - 2016
//
// Copyright: See README file that comes with this source code
//
//

#ifndef TERMTABLE_H
#define TERMTABLE_H

#include "tablewindow.h"

#include <QAbstractTableModel>
#include <QDialog>


class Molecule;
class IsoTab;

class QListWidget;
class QRadioButton;

struct Perturbation;

class TermData : public QAbstractTableModel
{
	public:
		TermData(QObject *parent = 0);
		~TermData();
		int rowCount(const QModelIndex &parent = QModelIndex()) const;
		int columnCount(const QModelIndex &parent = QModelIndex()) const;
		QVariant data (const QModelIndex &index, int role = Qt::DisplayRole) const;
		QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
		double ****getData();
		void setData(double ****nData, int numComp, int numIso, int maxv, int maxJ,
					 int *CompZ = 0, int nStates = 0, double *****nMixKoeff = 0);
		void setIso(IsoTab *Iso);
		IsoTab *getIso();
		void setIsoZ(int *Z);
		void GetIsoZ(int IsoI, int &NIso1, int &NIso2);
		int *getCompZ();
		int getNumComp();
		int getNumIso();
		int getMaxJ();
		int getMaxv();
		void getRows(int C, int I, int v, int *J, int N, int *R);
		void getJE(int *R, int N, int *J, double *E);
		
		inline int *getIsoZ()
		{
			return Z;
		}
		
		inline int getnumStates()
		{
			return numStates;
		}
		
		inline double *****getMixCoeff()
		{
			return MixCoeff;
		}
		
	private:
		int numRows, numv, numJ, numIso, numComp, *Z, ***vStart, *CompZ, numStates;
		double ****Data, *****MixCoeff;
		IsoTab *Iso;
};

class TermTable : public TableWindow
{
public:
    TermTable(MainWindow *MW = 0);
    ~TermTable();
	bool readData(QString Filename);
	bool writeData(QString Filename = "");
	double ****getData();
	int getNumComp();
	int getNumIso();
	int getMaxv();
	int getMaxJ();
	ElState *getElState();
	void setMolecule(Molecule *mol, ElState *nState);
	void setData(double ****nData, int numComp, int numIso, int maxv, int maxJ, int *CompZ = 0, int nStates = 0, double *****MixC = 0);
	void GetIsoZ(int IsoI, int &NIso1, int &NIso2);
	int *getIsoT();
	void getCompT(int &Max, int *&Trans);
	void getThermPopulation(double **&Pop, int &nv, int &nJ, double T, int Iso, int Comp);
	int getNPerturbations(int c = -1, int Iso = -1, int v = -1);
	Perturbation *getPerturbation(int c, int Iso, int v, int Number);
	Perturbation *getPerturbation(int N);
	void setPerturbations(int NPerturbations, Perturbation* NewPerturbations);
	virtual void setName(QString name);
	void getViewnE(int*& Js, double*& E, int& N);
	void getSelE(int*& Js, double*& E, int &N);
	void setViewnLevels(MDIChild *Viewer, int C, int I, int v, int *J, int N);
	void getMaxAndAverageDeviation(TermTable *Other, int comp, int iso, int vMax, 
								   int JMax, double &MaxDev, double &AvDev);
	
	inline int *getCompZ()
	{
		return tDat->getCompZ();
	}
	
	inline int *getIsoZ()
	{
		return tDat->getIsoZ();
	}
	
	inline void setIsoZ(int *Z)
	{
		tDat->setIsoZ(Z);
	}
	
	inline int getNumLines()
	{
		return tDat->rowCount();
	}
	
	inline int getNumStates()
	{
		return tDat->getnumStates();
	}
	
	inline double*****getMixCoefficients()
	{
		return tDat->getMixCoeff();
	}
	
protected:
	TermData *tDat;
	
private:
	bool OpenXnfitTO(QString Dateiname);
	void AssignIso();
	ElState *State;
	int NPerturbations;
	Perturbation *Perturbations;
};

struct Perturbation
{
	int Comp, Iso, v, J, Pv, PComp;
	TermTable *Perturber;
	QString PName;
};

class CTTIDialog : public QDialog
{
	Q_OBJECT
	
public:
	CTTIDialog(MainWindow *MW);
	void getData(Molecule *&Mol, ElState **&States, int &NStates, int &Mv, int &MJ, bool &singleTable);

private slots:
	void molChanged(int i);
	void add();
	void remove();
	
private:
	
	MainWindow *mw;
	Molecule *mol;
	QListWidget *SList;
	QComboBox *SBox, *MolBox;
	QPushButton *Add, *Remove, *OK, *Cancel;
	QLineEdit *MaxJ, *Maxv;
	QRadioButton *STable, *MTables;
};

class MeasuredTermEnergies : public TableWindow
{
	public:
		MeasuredTermEnergies(MainWindow *MW = 0);
		bool readData(QString FileName);
		bool writeData(QString Filename = "");
		double ****getData();
		void setMolecule(Molecule *mol);
		void addData(int Iso, int Comp, int nTerm, double **nData, int **Ass);
		
	private:
		double ****Data;
		int NumComp, NumIso, Maxv, numVRows, MaxJ;
};

#endif
