//
// C++ Implementation: tools
//
//
// Author: Alexander Stein <AlexanderStein@t-online.de>, (C) 2006 - 2016
//
// Copyright: See README file that comes with this source code
//
//

#include "tools.h"
#include "elstate.h"
#include "molecule.h"
#include "utils.h"

#include <math.h>
#include <stdio.h>

#include <qmessagebox.h>

Progressions::Progressions() 
{ 
    P = NULL; 
}

Progressions::~Progressions() 
{ 
    Clear(); 
}

FLI::FLI(int n, int g)
{
    vs = new int[N=n];
    vss = new int[N];
    Jss = new int[N];
    Js = new int[N];
    DD = new double[N];
    Iso = new int[N];
	SNR = new double[N];
	oc = new double[N];
	lState = new ElState*[N];
	uState = new ElState*[N];
	satellite = new bool[N];
	overlap = new bool[N];
    G = g;
}

void Progressions::Insert(Marker **marker, const int &N, const int &G)
{
    //printf("Progressions::Insert N=%d, G=%d\n", N, G);
	FLI *NFLI = new FLI(N, G);
    NFLI->marker = marker;
	int i;
    for (i=0; i<N; i++)
    {
		NFLI->vss[i] = marker[i]->vss;
		NFLI->Jss[i] = marker[i]->Jss;
		NFLI->DD[i] = marker[i]->DD;
		NFLI->SNR[i] = marker[i]->SNR;
		NFLI->Iso[i] = marker[i]->Iso;
		NFLI->vs[i] = marker[i]->vs;
		NFLI->Js[i] = marker[i]->Js;
		NFLI->oc[i] = marker[i]->oc;
		NFLI->lState[i] = marker[i]->LState;
		NFLI->uState[i] = marker[i]->UState;
		NFLI->satellite[i] = marker[i]->satellite;
		NFLI->overlap[i] = marker[i]->overlap;
    }
    Insert(NFLI);
}

void Progressions::Insert(const IntProg* Prog, const int& G)
{
	FLI *NFLI = new FLI(Prog->N, G);
	int i;
	NFLI->marker = new Marker*[Prog->N];
	for (i=0; i < Prog->N; i++)
	{
		NFLI->marker[i] = Prog->marker[i];
		NFLI->vss[i] = Prog->vss[i];
		NFLI->Jss[i] = Prog->Jss[i];
		NFLI->DD[i] = Prog->DD[i];
		NFLI->SNR[i] = Prog->SNR[i];
		NFLI->Iso[i] = Prog->Iso;
		NFLI->vs[i] = Prog->vs;
		NFLI->Js[i] = Prog->Js;
		NFLI->oc[i] = Prog->oc[i];
		NFLI->lState[i] = Prog->lState;
		NFLI->uState[i] = Prog->uState;
		NFLI->satellite[i] = Prog->satellite;
		NFLI->overlap[i] = Prog->overlap[i];
	}
	Insert(NFLI);
}

void Progressions::Insert(FLI* NFLI)
{
	if (P == NULL) 
    {
		P = NFLI;
		P->First = P->Next = NULL;
    }
    else 
    {
		//printf("Vor While\n");
		while(P->First != NULL) P = P->First;
	   	 //printf("Vor For\n");
		for (; P->G > NFLI->G && P->Next != NULL; P = P->Next) ;
		if (P->G > NFLI->G)
		{
		    //printf("Pos1\n");
	    	P->Next = NFLI;
	    	NFLI->Next = NULL;
	    	NFLI->First = P;
		}
		else
		{
		    //printf("Pos2\n");
		    NFLI->Next = P;
	    	NFLI->First = P->First;
	    	P->First = NFLI;
	    	if (NFLI->First!=NULL) NFLI->First->Next = NFLI;
	    	P = NFLI;
		}
		while(P->First != NULL) P = P->First;
    }
}
    
void Progressions::RotateB()
{
    if (P!=NULL) if (P->First != NULL) P = P->First;
}

void Progressions::RotateF()
{
    if (P!=NULL) if (P->Next != NULL) P = P->Next;
}

void Progressions::GetMarker(Marker **&marker, int &N)
{
    //printf("Progressions::GetMarker\n");
	if (P==NULL) 
    {
		marker = NULL;
		N = 0;
		return;
    }
    N = P->N;
    marker = P->marker;
    int i;
	IsoTab *Iso = 0;
	Molecule *Mol = 0;
    for (i=0; i<N; i++)
    {
		marker[i]->Marked = true;
		marker[i]->DisplayData = true;
		marker[i]->added = false;
		marker[i]->vss = P->vss[i];
		marker[i]->Jss = P->Jss[i];
		marker[i]->DD = P->DD[i];
		marker[i]->SNR = P->SNR[i];
		marker[i]->Iso = P->Iso[i];
		marker[i]->vs = P->vs[i];
		marker[i]->Js = P->Js[i];
		marker[i]->satellite = P->satellite[i];
		marker[i]->oc = P->oc[i];
		marker[i]->lState = ((marker[i]->LState = P->lState[i]) != 0 ? marker[i]->LState->getName() : "");
		marker[i]->uState = ((marker[i]->UState = P->uState[i]) != 0 ? marker[i]->UState->getName() : "");
		marker[i]->Mol = (marker[i]->LState != 0 ? marker[i]->LState->getMolecule() 
			                 : (marker[i]->UState != 0 ? marker[i]->UState->getMolecule() : 0));
		if (marker[i]->Mol != 0 && marker[i]->Mol != Mol) Iso = (Mol = marker[i]->Mol)->getIso();
		marker[i]->IsoName = (Iso != 0 ? Iso->texName[marker[i]->Iso] : "");
		marker[i]->overlap = P->overlap[i];
		marker[i]->FC = -1;
		marker[i]->uncertainty = (marker[i]->overlap ? 0.01 : 0.005);
    }
	//printf("Ende GetMarker\n");
}

void Progressions::Clear()
{
    if (P==NULL) return;
    while (P->First != NULL) P = P->First;
    FLI *F;
    while (P != NULL)
    {
		delete[] P->marker;
		delete[] P->vss;
		delete[] P->Jss;
		delete[] P->DD;
		delete[] P->Iso;
		delete[] P->Js;
		delete[] P->vs;
		delete[] P->SNR;
		delete[] P->lState;
		delete[] P->uState;
		delete[] P->satellite;
		delete[] P->overlap;
		delete[] P->oc;
		F = P;
		P = P->Next;
		delete F;
    }
}

Datensatz::Datensatz()
{
    //printf("Datensatz::Datensatz()\n");
    AE = new Element;
    AE->First = AE->Next = NULL;
    AE->minIndex = 0;
    AE->maxIndex = 999;
    AIndex = 0;
    G = 1000;
    return;
}
    
Datensatz::~Datensatz()
{
    //printf("Datensatz::~Datensatz()\n");
    while (AE->Next != NULL) AE = AE->Next;
    //printf("Start delete, G=%d\n", G);
	//int i=0;
	while (AE->First != NULL)
    {
		//printf("delete Element %d\n", i++);
		AE = AE->First;
		delete AE->Next;
    }
	//printf("delete last element\n");
    delete AE;
	//printf("Ende ~Datensatz\n");
    return;
}

void Datensatz::reinit()
{
    while (AE->Next != NULL) AE = AE ->Next;
    while (AE->First != NULL)
    {
	AE = AE->First;
	delete AE->Next;
    }
    AE->Next = NULL;
    G = 1000;
    AIndex = 0;
}

int Datensatz::GetDSL() const
{
    return G - 1000 + AIndex;
}
    
void Datensatz::AddValue(const double &x, const double &y, const bool marked)
{
    if (AIndex == 1000)
    {
		AIndex = 0;
		G+=1000;
		AE->Next = new Element;
		AE->Next->First = AE;
		AE->Next->Next = NULL;
		AE->Next->minIndex = AE->minIndex + 1000;
		AE->Next->maxIndex = AE->maxIndex + 1000;
		AE = AE->Next;
    }
    AE->Daten[AIndex][0] = x;
    AE->Daten[AIndex][1] = y;
    AE->marker[AIndex] = NULL;
    if (marked) AE->marker[AIndex] += 1;
    AIndex++;
}
    
void Datensatz::InsertML(Datensatz &ML)
{
    //printf("Insert erreicht\n");
    int i, EIndex=0, MLIndex=0, NML = ML.GetDSL(), 
    N = GetDSL();
    int GN = NML + N;
    Element *New = NULL, *Next = NULL; 
    while (AE->First != NULL) AE = AE->First;
    AIndex = 999;
    G = 0;
    double Nx = ML.GetValue(0, 0), Ax = AE->Daten[0][0];
    //printf("NML=%d, N=%d\n", NML, N);
    //printf("Nx=%f; Ax=%f\n", Nx, Ax);
    for (i=0; i<GN; i++)
    {
		if (AE == NULL)
		{
	    	printf("Fehler: zu große X-Werte der Marker\n");
	    	break;
		}
		Ax = AE->Daten[EIndex][0];
		if (++AIndex == 1000)
		{
	    	//printf("A1000 i=%d\n", i);
	    	if (Next == NULL)
	    	{
				if (New == NULL)
				{
		    		New = new Element;
		    		New->First = NULL;
				}
				else
				{
		    		New->Next = new Element;
		    		New->Next->First = New;
		    		New = New->Next;
				}
	    	}
	    	else
	    	{
				Next->First = New;
				New->Next = Next;
				New = Next;
				Next = NULL;
	    	}
	    	New->minIndex = G;
	    	New->maxIndex = (G += 1000) - 1;
	    	AIndex = 0;
		}
		if (EIndex < N ? (MLIndex < NML ? Nx < Ax : false) : true)
		{
	    	New->Daten[AIndex][0] = Nx;
	    	New->Daten[AIndex][1] = ML.GetValue(MLIndex, 1);
	    	//printf("Daten[%d][0]=%f; Daten[%d][1]=%f\n", AIndex, New->Daten[AIndex][0],
	    	//     AIndex, New->Daten[AIndex][1]);
	    	New->marker[AIndex] = ML.GetMarker(MLIndex++);
	    	//if (New->marker[AIndex] == NULL) 
			//	printf("Ungemarkerte Linie bei AIndex=%d, MLIndex=%d\n", AIndex, MLIndex - 1);
	    	New->marker[AIndex]->Line = New->Daten[AIndex];
	    	if (MLIndex < NML) Nx = ML.GetValue(MLIndex, 0);
		}
		else
		{
	    	New->Daten[AIndex][0] = Ax;
	    	New->Daten[AIndex][1] = AE->Daten[EIndex][1];
	    	New->marker[AIndex] = AE->marker[EIndex++];
	    	if (EIndex == 1000)
	    	{
				//printf("EIndex1000 i=%d, APos=%d\n", i, AE->minIndex+EIndex);
				Next = AE;
				AE = AE->Next;
				EIndex = 0;
	    	}
		}
    }
    //printf("MLIndex=%d, EIndex=%d, AIndex=%d\n", MLIndex, EIndex, AIndex);
	//printf("Nach Schleife\n");
    AIndex++;
	if (Next != NULL) delete Next;
    AE = New;
    AE->Next = NULL;
    //printf("Am Ende von InserML\n");
}

void Datensatz::ReverseOrder()
{
    //printf("ReverseOrder\n");
    Element *NF = new Element;
    NF->First = NULL;
    NF->minIndex = 0;
    NF->maxIndex = 999;
    int i, N = GetDSL(), NIndex = -1, NG = 1000;
    while (AE->Next != NULL) AE = AE->Next;
    for (i=0; i<N; i++)
    {
	if (--AIndex < 0)
	{
	    NF->Next = AE;
	    AE->minIndex = NG;
	    //printf("NG=%d\n", NG);
	    AE->maxIndex = (NG += 1000) - 1;
	    AE = AE->First;
	    NF->Next->First = NF;
	    AIndex = 999;
	}
	if (++NIndex == 1000)
	{
	    NIndex = 0;
	    NF = NF->Next;
	}
	NF->Daten[NIndex][0] = AE->Daten[AIndex][0];
	NF->Daten[NIndex][1] = AE->Daten[AIndex][1];
	NF->marker[NIndex] = AE->marker[AIndex];
    }
    NF->Next = NULL;
    AIndex = NIndex;
    AE->Next = NULL;
    AE->First = NULL;
    delete AE;
    AE = NF;
}

void Datensatz::SetMarker(int i, Marker *nmarker)
{
    //if (i>333000) printf("Daten.SetMarker, i=%d\n", i);
    while (AE->minIndex > i) AE = AE->First;
    while (AE->maxIndex < i) AE = AE->Next;
    i-=AE->minIndex;
    AE->marker[i] = nmarker;
    nmarker->Line = AE->Daten[i];
    nmarker->Visible = false;
    nmarker->Marked = false;
    nmarker->DisplayData = false;
	nmarker->satellite = false;
	nmarker->overlap = false;
	nmarker->oc = 0.0;
	nmarker->HFLM = -1;
}
    
Marker *Datensatz::GetMarker(int i)
{
	if (i < 0 || i >= G)
	{
		printf("Datensatz::GetMarker error: i=%d, G=%d\n", i, G);
		return 0;
	}
    while (AE->minIndex > i) AE = AE->First;
    while (AE->maxIndex < i) AE = AE->Next;
    i-=AE->minIndex;
    return AE->marker[i];
}

double Datensatz::GetValue(const int &i0, const int &i1)
{
    if (i0 < 0 || i0 >= G || G==0)
	{
		printf("Datensatz::GetValue error: i0=%d, G=%d\n", i0, G);
		return 0.0;
	}
	int i=i0;
    //if (DebugPrint) printf("GetValue, AE->minIndex=%d, i0=%d\n", AE->minIndex, i0);
    while (AE->minIndex > i0) AE = AE->First;
    /*if (i0==285000) 
{
    printf("GetValue: beim Beginn, i0=%d\n", i0);
    AE = AE->Next;
    printf("minIndex=%d, maxIndex=%d\n", AE->minIndex, AE->maxIndex);
}*/
    while (AE->maxIndex < i0) AE = AE->Next;
    i-=AE->minIndex;
    //if (DebugPrint) printf("GetValue: vor return,i=%d, i1=%d\n",i,i1);
    return AE->Daten[i][i1];
}

double *Datensatz::GetPoint(int i)
{
	if (i < 0 || i >= G)
	{
		printf("Datensatz::GetPoint error: i=%d, G=%d\n", i, G);
		return 0;
	}
	while (AE->minIndex > i) AE = AE->First;
	while (AE->maxIndex < i) AE = AE->Next;
	i -= AE->minIndex;
	return AE->Daten[i];
}

bool Datensatz::GetMarked(int i)
{
	if (i < 0 || i >= G)
	{
		printf("Datensatz::GetMarked error: i=%d, G=%d\n", i, G);
		return false;
	}
    while (AE->minIndex > i) AE = AE->First;
    while (AE->maxIndex < i) AE = AE->Next;
    i -= AE->minIndex;
    if (AE->marker[i] == NULL) return false;
    return true;
}

bool SolvLinEqS(double **G, int n)
{
	int i, j, k;
	double *B, F, T;
	for (i=0; i<n-1; i++)
	{
		for (B=G[0]; B != NULL;) 
		{
			B = NULL;
			for (j=i+1; j<n; j++) if (fabs(G[j-1][i]) < fabs(G[j][i]))
			{
				B = G[j-1];
				G[j-1] = G[j];
				G[j] = B;
			}
		}
		for (j=n-1; G[j][i]==0.0 && j>i; j--);
		if (j==i) return false;
		for (j--; j>=i; j--)
		{
			F = G[j+1][i] / G[j][i];
			for (k=i+1; k<=n; k++)
			{
				T = F * G[j][k];
				G[j+1][k] -= T;
				//if (fabs(G[j+1][k]) < fabs(Err * T)) G[j+1][k] = 0.0;
			}
		}
	}
	if (G[n-1][n-1] == 0.0) return false;
	for (i=n-1; i>=0; i--) 
	{
		for (j=n-1; j>i; j--) G[i][n] -= G[j][n] * G[i][j];
		G[i][n] /= G[i][i];
	}
	return true;
}	

bool SolvLinEqSwItImp(double** EqnS, int N, double *mFQS)
{
	double **tEqnS = Create(N, N+1), FQS = 1e98, aFQS = 1e99, tRes[N], bRes[N];
	int n, m;
	for (n=0; n<N; n++)
	{
		tEqnS[n][N] = EqnS[n][N];
		tRes[n] = 0.0;
	}
	while (FQS < aFQS)
	{
		aFQS = FQS;
		for (n=0; n<N; n++) for (m=0; m<N; m++) tEqnS[n][m] = EqnS[n][m];
		if (!SolvLinEqS(tEqnS, N)) return false;
		for (n=0; n<N; n++) tRes[n] += tEqnS[n][N];
		for (n=0, FQS = 0.0; n<N; n++)
		{
			for (tEqnS[n][N] = EqnS[n][N], m=0; m<N; m++) tEqnS[n][N] -= EqnS[n][m] * tRes[m];
			FQS += tEqnS[n][N] * tEqnS[n][N];
		}
		if (FQS < aFQS) for (n=0; n<N; n++) bRes[n] = tRes[n];
		printf("SolvLinEqSwItImp: FQS=%g\n", FQS);
	}
	for (n=0; n<N; n++) EqnS[n][N] = bRes[n];
	Destroy(tEqnS, N);
	if (mFQS != 0) *mFQS = aFQS;
	return true;
}

int Runden(const double &d)
{
    int i = (int)d;
    if (d - (double)i > 0.5) return i+1;
    return i;
}

void Test(double &d1, double &d2, QLineEdit *L1, QLineEdit *L2)
{
    double buffer;
    d1 = L1->text().toDouble();
    d2 = L2->text().toDouble();
    if (d1 >= d2)
    {
	if (d1 > d2)
	{
	    buffer = d1;
	    d1 = d2;
	    d2 = buffer;
	}
	else 
	{
	    d1 -= 0.5;
	    d2 += 0.5;
	}
	L1->setText(QString::number(d1, 'g', 11));
	L2->setText(QString::number(d2, 'g', 11));
    }
}

void GSearchLines(Marker *marker, Marker *LaserLine, const int &AnzahlMarker, const double &ST,
		  const double &GMH, Progressions &Prog, const int &veu, const int &Jeu, const int &NI,
		  double ***&ELU)
{
    Marker *FoundLines[veu+1];
    Marker **MB, *LL;
    int FoundCount = 0, i, j=0, FG=0, AGMarker = 0, Ii, viu, Jiu;
    for (i=0; i<AnzahlMarker; i++) 
    {
	if (marker[i].Marked) 
	{
	    j++;
	    FG = i;
	}
	if (marker[i].HFLM >= GMH) AGMarker++; 
    }
    if (j==1) LL = marker + FG;
    else LL = LaserLine;
    Marker *GMarker[AGMarker];
    j = 0;
    for (i=0; i<AnzahlMarker; i++) if (marker[i].HFLM >= GMH) GMarker[j++] = marker + i;
    double LES, ZS;
    for (Ii=0; Ii < NI; Ii++) 
    {
	for (Jiu=0; Jiu < Jeu; Jiu++) 
	{
	    for (viu=0; viu < veu; viu++)
	    {
		//printf("Ii=%d, Jiu=%d, viu=%d\n", Ii, Jiu, viu);
		if(ELU[Ii][viu][Jiu] == 0.0) break;
		LES = LL->Line[0] + ELU[Ii][viu][Jiu];
		j = AGMarker - 2;
		for (i=0; i<veu; i++)
		{
		    if(ELU[Ii][i][Jiu] == 0.0) break;
		    ZS = LES - ELU[Ii][i][Jiu];
		    while (ZS < GMarker[j]->Line[0] && j>0) j--;
		    if (GMarker[j+1]->Line[0] - ZS < ZS - GMarker[j]->Line[0]) j++;
		    if (fabs(GMarker[j]->Line[0] - ZS) < ST) FG++;
		    if (j==AGMarker-1) j--;
		} 
		if (FG > 2)
		{
		    printf("Progression gefunden.\n");
		    j = AnzahlMarker - 2;
		    for (i=0; i<veu; i++)
		    {
			if(ELU[Ii][i][Jiu] == 0.0) break;
			ZS = LES - ELU[Ii][i][Jiu];
			while (ZS < marker[j].Line[0] && j>0) j--;
			if (marker[j+1].Line[0] - ZS < ZS - marker[j].Line[0]) j++;
			if (fabs(marker[j].Line[0] - ZS) < ST)
			{
			    //printf("Gefunden: v=%d, J=%d, WN=%f\n", i, Jiu, marker[j].Line[0]);
			    FoundLines[FoundCount] = marker + j;
			    FoundLines[FoundCount]->DD = ZS - FoundLines[FoundCount]->Line[0];
			    FoundLines[FoundCount]->Iso = Ii;
			    FoundLines[FoundCount]->vss = i;
			    FoundLines[FoundCount++]->Jss = Jiu;
			}
			if (j==AnzahlMarker-1) j--;
		    } 
		    MB = new Marker*[FoundCount];
		    for (i=0; i<FoundCount; i++) MB[i] = FoundLines[i];
		    Prog.Insert(MB, FoundCount, FG);
		    printf("FoundCount=%d\n", FoundCount);
		}
		FoundCount = FG = 0;
	    }
	}
    }
}

IntProg::IntProg()
{
	clear();
	marker = new Marker*[1000];
}

void IntProg::copy(const IntProg &C)
{
	//printf("Intprog::CConstructor\n");
	int n;
	for (n=0; n < C.N; n++)
	{
		marker[n] = C.marker[n];
		DD[n] = C.DD[n];
		Jss[n] = C.Jss[n];
		oc[n] = C.oc[n];
		overlap[n] = C.overlap[n];
		SNR[n] = C.SNR[n];
		vss[n] = C.vss[n];
	}
	FQS = C.FQS;
	G = C.G;
	Iso = C.Iso;
	Js = C.Js;
	N = C.N;
	NGD = C.NGD;
	Np = C.Np;
	vs = C.vs;
	satellite = C.satellite;
	lState = C.lState;
	uState = C.uState;
	Mol = C.Mol;
	QF = C.QF;
	FC = C.FC;
}

bool IntProg::isbetter(IntProg *C)
{
	if (uState != 0 && C->uState == 0) return true;
	if (uState == 0 && C->uState != 0) return false;
	if (satellite && !C->satellite) return true;
	if (!satellite && C->satellite) return false;
	if (NGD > C->NGD) return true;
	if (NGD < C->NGD) return false;
	if (G > C->G) return true;
	if (G < C->G) return false;
	if (N > C->N) return true;
	if (N < C->N) return false;
	if (FQS < C->FQS) return true;
	return false;
}

bool IntProg::isIncluded(Marker *M)
{
	int i;
	for (i=0; i<N; i++) if (M == marker[i]) return true;
	return false;
}

IntProg::~IntProg()
{
	//printf("Vor delete marker\n");
	if (marker != NULL) delete[] marker;
	//printf("Nach delete marker\n");
}

void IntProg::clear()
{
	N = G = NGD = vs = Js = Iso = 0;
	FC = -1;
	satellite = false;
	FQS = 0.0;
	lState = uState = 0;
	Mol = 0;
}

void ParabInterpol(double x1, double y1, double x2, double y2, double x3, double y3, double &c, double &a)
{
	double b, xd21, xd32, x1q, x2q, x3q, dxq12, dxq32, dq, yd12, yd23, dxc;
	xd21 = x2 - x1;
	xd32 = x3 - x2;
	x1q = x1 * x1;
	x2q = x2 * x2;
	x3q = x3 * x3;
	dxq12 = x1q - x2q;
	dxq32 = x3q - x2q;
	dq = xd21 / xd32;
	yd12 = y1 - y2;
	yd23 = y2 - y3;
	b = (yd12 - yd23 * dq) / (dxq12 + dxq32 * dq);
	c = (yd23 + b * dxq32) / (2 * b * xd32);
	dxc = x1 - c;
	a = y1 - b * dxc * dxc;
}

