//
// C++ Interface: tools
//
//
// Author: Alexander Stein <AlexanderStein@t-online.de>, (C) 2006 - 2016
//
// Copyright: See README file that comes with this source code
//
//

#ifndef TOOLS_H
#define TOOLS_H

#include <qlineedit.h>

class ElState;
class Molecule;

struct Marker
{
    const double *Line, *LMin, *RMin;
    bool Marked, Visible, DisplayData, satellite;
    int x1, y1, x2, y2, vss, Jss, Iso, vs, Js, FC;
    double DD, SNR, oc, HFLM, uncertainty;
	bool overlap, sOverlap, b, added;
	QString IsoName, lState, uState;
	ElState *LState, *UState;
	Molecule *Mol;
};

struct FLI
{
    FLI(int N, int G);
	
	Marker **marker;
    FLI *Next, *First;
    int *vss, *Jss, *vs, *Js, *Iso, N, G;
    double *DD, *SNR, *oc;
	bool *satellite, *overlap;
	ElState **lState, **uState;
};

struct Band
{
	Marker *lines[1000];
	int MinI, MaxI, CL1, CL2;
};

struct FittetLine
{
    double Position, Width, Height;
};

struct IntProg
{
	IntProg();
	~IntProg();
	void clear();
	void copy(const IntProg&);
	bool isbetter(IntProg*);
	bool isIncluded(Marker*);
	
	int N, G, NGD, Iso, Np;
	Marker **marker;
	bool satellite, overlap[1000];
	int vss[1000], Jss[1000], vs, Js, FC;
	double FQS, uT, oc[1000], DD[1000], SNR[1000], QF;
	ElState *lState, *uState;
	Molecule *Mol;
};

class Progressions
{
public:
    Progressions();
    ~Progressions();
	void Insert(const IntProg *Prog, const int &G);
    void Insert(Marker **marker, const int &N, const int &G);
    void RotateB();
    void RotateF();
    void GetMarker(Marker **&marker, int &N);
    void Clear();

private:
	void Insert(FLI *newElement);
	
    FLI *P;
};

struct Line
{
	int vss, Jss, row;
	double E, err;
};

struct Progression
{
	int N, vs, Js, Iso;
	Line *L;
};

struct Element
{
    Element *First;
    Element *Next;
    double Daten[1000][2];
    Marker *marker[1000];
    int minIndex;
    int maxIndex;
};

class Datensatz      //Einlesefunktion nur für Einlesen in einem Stück gedacht!
{
public:    
    Datensatz();
    ~Datensatz();
    void reinit();
    int GetDSL() const;
    void AddValue(const double &x, const double &y, const bool marked);
    void InsertML(Datensatz &ML);
    void ReverseOrder();
    void SetMarker(int i, Marker *nmarker);
    Marker *GetMarker(int i);
    double GetValue(const int &i0, const int &i1);
    bool GetMarked(int i);
	double *GetPoint(int i);
private:
    Element *AE;
    int AIndex, G;
};

struct TestProg
{
    int *vss, *Jss, *Iso, *vs, *Js;
    double *DD, SBD, SDiff, oc;
	ElState **lState, **uState;
};

struct SortProg
{
	int G, N, Iso;
	double SDiff;
	Marker **marker;
	SortProg *next;
	int *vss, *Jss, *vs, *Js;
	double *Diff;
};

struct AProg
{
	int N, *J, *v;
	QString ULevel;
	double *Freq;
};

void ParabInterpol(double x1, double y1, double x2, double y2, double x3, double y3, double &x, double &y);
bool SolvLinEqS(double **EqnS, int n);
bool SolvLinEqSwItImp(double **EqnS, int n, double *mFQS = 0);
int Runden(const double &d);
void Test(double &d1, double &d2, QLineEdit *L1, QLineEdit *L2);
void GSearchLines(Marker *marker, Marker *LaserLine, const int &AnzahlMarker, const double &ST,
		  const double &GMH, Progressions &Prog, const int &veu, const int &Jeu, const int &NI,
		  double ***&ELU);

#endif
