//
// C++ Implementation: utils
//
//
// Author: Alexander Stein <AlexanderStein@t-online.de>, (C) 2006 - 2016
//
// Copyright: See README file that comes with this source code
//
//

#include "utils.h"

#include <stdio.h>

#include <QString>

DoubleVector::DoubleVector(int start) : FlexVector<double>(start)
{
	int i;
	for (i=0; i< 1000; i++) CurElement->Data[i] = 0.0;
}

double &DoubleVector::operator[](const int &i)
{
	while (CurElement->startI > i)
	{
		if (CurElement->prev == 0)
		{
			int j;
			CurElement->prev = new VElement<double>;
			CurElement->prev->next = CurElement;
			CurElement->prev->prev = 0;
			CurElement->prev->startI = CurElement->startI - 1000;
			CurElement->prev->endI = CurElement->startI - 1;
			for (j=0; j < 1000; j++) CurElement->prev->Data[j] = 0.0;
		}
		CurElement = CurElement->prev;
	}
	while (CurElement->endI < i)
	{
		if (CurElement->next == 0)
		{
			int j;
			CurElement->next = new VElement<double>;
			CurElement->next->prev = CurElement;
			CurElement->next->next = 0;
			CurElement->next->startI = CurElement->endI + 1;
			CurElement->next->endI = CurElement->endI + 1000;
			for (j=0; j < 1000; j++) CurElement->next->Data[j] = 0.0;
		}
		CurElement = CurElement->next;
	}
	return CurElement->Data[i - CurElement->startI];
}


IsoTab::IsoTab(int NumIso)
{
	numIso = NumIso;
	mNumIso1 = new int[numIso];
	mNumIso2 = new int[numIso];
	relNA = new double[numIso];
	redMass = new double[numIso];
	relRedMass = new double[numIso];
	rootRRM = new double[numIso];
	texName = new QString[numIso];
	JStep = new int[numIso];
	mIso1 = new double[numIso];
	mIso2 = new double[numIso];
	chSymb1 = new QString;
	chSymb2 = new QString;
}

IsoTab::~IsoTab()
{
	//printf("IsoTab::~IsoTab()\n");
	delete[] mNumIso1;
	delete[] mNumIso2;
	delete[] relNA;
	delete[] redMass;
	delete[] relRedMass;
	delete[] rootRRM;
	delete[] texName;
	delete[] JStep;
	delete[] mIso1;
	delete[] mIso2;
	delete chSymb1;
	delete chSymb2;
	//printf("Ende ~IsoTab\n");
}

QString IsoTab::getIsoName(int N)
{
	if (N < 0 || N >= numIso) return "";
	if (mNumIso1[N] == mNumIso2[N] && *chSymb1 == *chSymb2) return QString::number(mNumIso1[N]) + *chSymb1 + "2";
	return QString::number(mNumIso1[N]) + *chSymb1 + QString::number(mNumIso2[N]) + *chSymb2;
}


QString **CreateQString(const int &I, const int &J)
{
	QString **R = new QString*[I];
	int i;
	for (i=0; i<I; i++) R[i] = new QString[J];
	return R;
}


void Destroy(QString **v, const int &I)
{
	int i;
	for (i=0; i<I; i++) delete[] v[i];
	delete[] v;
}


