//
// C++ Interface: utils
//
//
// Author: Alexander Stein <AlexanderStein@t-online.de>, (C) 2006 - 2016
//
// Copyright: See README file that comes with this source code
//
//

#ifndef UTILS
#define UTILS

#define sqr(x) ((x)*(x))

class QString;

template <class T> struct VElement
{
	VElement *next, *prev;
	int startI, endI;
	T Data[1000];
};

template <class T> class FlexVector
{
	public:
		FlexVector(int start = -499);
		~FlexVector();
		T &operator[](const int &i);
		T *getData();
		void clear();
		int getFirstIndex();
		int getLastIndex();
		
	protected:
		VElement<T> *CurElement;
		int FI, LI;
};

class DoubleVector : public FlexVector<double>
{
	public:
		DoubleVector(int start = -499);
		double &operator[](const int &i); 
};

template <class T> FlexVector<T>::FlexVector(int start)
{
	CurElement = new VElement<T>;
	CurElement->next = CurElement->prev = 0;
	CurElement->startI = start;
	CurElement->endI = start + 999;
	FI = LI = start;
}

template <class T> FlexVector<T>::~FlexVector()
{
	clear();
}

template <class T> void FlexVector<T>::clear()
{
	while (CurElement->next != 0) CurElement = CurElement->next;
	while (CurElement->prev != 0) 
	{
		CurElement = CurElement->prev;
		delete CurElement->next;
	}
	delete CurElement;
}

template <class T> T *FlexVector<T>::getData()
{
	int i, j, L = FI - LI + 1;
	T *R = new T[L];
	while (CurElement->startI > FI) CurElement = CurElement->prev;
	for (i=0, j = FI - CurElement->startI; j <= LI; i++, j++) 
	{
		if (j == 1000) 
		{
			CurElement = CurElement->next;
			j=0;
		}
		R[i] = CurElement->Data[j];
	}
	return R;
}

template <class T> int FlexVector<T>::getFirstIndex()
{
	return FI;
}

template <class T> int FlexVector<T>::getLastIndex()
{
	return LI;
}

template <class T> T &FlexVector<T>::operator[](const int &i)
{
	while (CurElement->startI > i)
	{
		if (CurElement->prev == 0)
		{
			CurElement->prev = new VElement<T>;
			CurElement->prev->next = CurElement;
			CurElement->prev->prev = 0;
			CurElement->prev->startI = CurElement->startI - 1000;
			CurElement->prev->endI = CurElement->startI - 1;
		}
		CurElement = CurElement->prev;
	}
	while (CurElement->endI < i)
	{
		if (CurElement->next == 0)
		{
			CurElement->next = new VElement<T>;
			CurElement->next->prev = CurElement;
			CurElement->next->next = 0;
			CurElement->next->startI = CurElement->endI + 1;
			CurElement->next->endI = CurElement->endI + 1000;
		}
		CurElement = CurElement->next;
	}
	return CurElement->Data[i - CurElement->startI];
}

template <class T> struct Matrix
{
	Matrix(const int &i, const int &j)
	{
		R = new T*[i];
		int n;
		for (n=0; n<i; n++) R[n] = new T[j];
		nr = i;
	}
	~Matrix()
	{
		int n;
		for (n=0; n<nr; n++) delete[] R[n];
		delete[] R;
	}
	int nr;
	T **R;
};

QString **CreateQString(const int &i, const int &j);
void Destroy(QString **v, const int &i);

inline double **Create(const int &I, const int &J)
{
	double **R = new double*[I];
	int i;
	for (i=0; i<I; i++) R[i] = new double[J];
	return R;
}

inline int **CreateInt(const int &I, const int &J)
{
	int **R = new int*[I];
	int i;
	for (i=0; i<I; i++) R[i] = new int[J];
	return R;
}

inline int ***CreateInt(const int &I, const int &J, const int &K)
{
	int ***R = new int**[I];
	int i, j;
	for (i=0; i<I; i++)
	{
		R[i] = new int*[J];
		for (j=0; j<J; j++) R[i][j] = new int[K];
	}
	return R;
}

inline bool **CreateBool(const int &I, const int &J)
{
	bool **R = new bool*[I];
	int i;
	for (i=0; i<I; i++) R[i] = new bool[J];
	return R;
}

inline double **Create1(const int &I, const int &J)
{
	double **R = new double*[I];
	int i;
	for (i=0; i<I; i++) R[i] = (new double[J]) - 1;
	return R - 1;
}

inline double ***Create(const int &I, const int &J, const int &K)
{
	double ***R = new double**[I];
	int i;
	for (i=0; i<I; i++) R[i] = Create(J, K);
	return R;
}

inline bool ***CreateBool(const int &I, const int &J, const int &K)
{
	bool ***R = new bool **[I];
	int i;
	for (i=0; i<I; i++) R[i] = CreateBool(J, K);
	return R;
}

inline double ****Create(const int &I, const int &J, const int &K, const int &L)
{
	double ****R = new double ***[I];
	int i;
	for (i=0; i<I; i++) R[i] = Create(J, K, L);
	return R;
}

inline bool ****CreateBool(const int &I, const int &J, const int &K, const int &L)
{
	bool ****R = new bool ***[I];
	int i;
	for (i=0; i<I; i++) R[i] = CreateBool(J, K, L);
	return R;
}

inline double***** Create(const int& i, const int& j, const int& k, const int& l, const int& m)
{
	double *****R = new double****[i];
	int n;
	for (n=0; n<i; n++) R[n] = Create(j, k, l, m);
	return R;
}

inline bool***** CreateBool(const int& i, const int& j, const int& k, const int& l, const int& m)
{
	bool *****R = new bool****[i];
	int n;
	for (n=0; n<i; n++) R[n] = CreateBool(j, k, l, m);
	return R;
}

inline void Destroy(double **v, const int &I)
{
	int i;
    //printf("Vor delete v[i]\n");
	for (i=0; i<I; i++) {
	    //printf("i=%d\n", i);
		delete[] v[i];
	}
    //printf("Vor delete v\n");
	delete[] v;
}

inline void Destroy(int **v, const int &I)
{
	int i;
	for (i=0; i<I; i++) delete[] v[i];
	delete[] v;
}

inline void Destroy(bool **v, const int &I)
{
	int i;
	for (i=0; i<I; i++) delete[] v[i];
	delete[] v;
}

inline void Destroy1(double **v, const int &I)
{
	int i;
	for (i=1; i<=I; i++) delete[] (v[i] + 1);
	delete[] (v + 1);
}

inline void Destroy(double ***v, const int &I, const int &J)
{
	int i;
	for (i=0; i<I; i++) Destroy (v[i], J);
	delete[] v;
}

inline void Destroy(bool ***v, const int &I, const int &J)
{
	int i;
	for (i=0; i<I; i++) Destroy(v[i], J);
	delete[] v;
}

inline void Destroy(int ***v, const int &I, const int &J)
{
	int i, j;
	for (i=0; i<I; i++) 
	{
		for (j=0; j<J; j++) delete[] v[i][j];
		delete[] v[i];
	}
	delete[] v;
}

inline void Destroy(double ****v, const int &I, const int &J, const int &K)
{
	int i;
	for (i=0; i<I; i++) Destroy(v[i], J, K);
	delete[] v;
}

inline void Destroy(bool ****v, const int &I, const int &J, const int &K)
{
	int i;
	for (i=0; i<I; i++) Destroy(v[i], J, K);
	delete[] v;
}

inline void Destroy(double***** v, const int& i, const int& j, const int& k, const int& l)
{
	int n;
	for (n=0; n<i; n++) Destroy(v[n], j, k, l);
	delete[] v;
}

inline void Destroy(bool***** v, const int& i, const int& j, const int& k, const int& l)
{
	int n;
	for (n=0; n<i; n++) Destroy(v[n], j, k, l);
	delete[] v;
}

inline int rund(double V)
{
	int R = int(V);
	double r = V - double(R);
	if (r >= 0.5) return ++R;
	if (r <= -0.5) return --R;
	return R;
}

class IsoTab
{
	public:
		IsoTab(int NumIso);
		~IsoTab();
		QString getIsoName(int N);
	
		QString *chSymb1, *chSymb2, *texName;
		int numIso, *mNumIso1, *mNumIso2, refIso, *JStep;
		double *relNA, *redMass, *relRedMass, *rootRRM, *mIso1, *mIso2;
};

#endif //UTILS
